
# src\file_conversor\cli\win\unins_cmd.py

import typer

from typing import Annotated

from rich import print


# user-provided modules
from file_conversor.backend import WinRegBackend

from file_conversor.cli.win._typer import CONTEXT_MENU_PANEL as RICH_HELP_PANEL
from file_conversor.cli.win._typer import COMMAND_NAME, UNINSTALL_MENU_NAME

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.system.win.ctx_menu import WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = WinRegBackend.EXTERNAL_DEPENDENCIES


@typer_cmd.command(
    name=UNINSTALL_MENU_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Uninstalls app context menu (right click in Windows Explorer).')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {UNINSTALL_MENU_NAME}` 
""")
def uninstall_menu():
    winreg_backend = WinRegBackend(verbose=STATE["verbose"])

    logger.info(f"{_('Removing app context menu from Windows Explorer')} ...")

    # Define registry path
    ctx_menu = WinContextMenu.get_instance()
    # logger.debug("---- .REG file contents ----")
    # logger.debug(repr(ctx_menu.get_reg_file()))

    winreg_backend.delete_keys(ctx_menu.get_reg_file())

    logger.info(f"{_('Context Menu Uninstall')}: [bold green]{_('SUCCESS')}[/].")




# src\file_conversor\config\__init__.py

"""
This module initializes the app configuration modules.
"""

from file_conversor.config.cleanup import add_cleanup_task
from file_conversor.config.config import Configuration
from file_conversor.config.environment import Environment
from file_conversor.config.locale import *
from file_conversor.config.log import Log
from file_conversor.config.state import State



# src/file_conversor/backend/gui/xls/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.xls._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def xls_index():
    tools = [
        {
            'image': url_for('icons', filename='convert.ico'),
            'title': _("Convert"),
            'subtitle': _("Convert spreadsheet files into other formats (requires Microsoft Office / LibreOffice)."),
            'url': url_for('xls_convert'),
        },
    ]
    return render_template_string(str(
        PageCardGrid(
            *tools,
            nav_items=[
                home_nav_item(),
                xls_index_nav_item(active=True),
            ],
            _title=f"{_('Spreadsheet Tools')} - File Conversor",
        )
    ))



# src\file_conversor\backend\audio_video\abstract_ffmpeg_backend.py


# user-provided imports
from typing import Any, Iterable
from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.backend.audio_video.format_container import AudioFormatContainer, VideoFormatContainer

from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class AbstractFFmpegBackend(AbstractBackend):
    """
    AbstractFFmpegBackend is a class that provides an interface for handling audio and video files using FFmpeg.
    """

    ENCODING_SPEEDS = ["fast", "medium", "slow"]
    QUALITY_PRESETS = ["high", "medium", "low"]

    SUPPORTED_IN_AUDIO_FORMATS = {
        'aac': {},
        'ac3': {},
        'flac': {},
        'm4a': {},
        'mp3': {},
        'ogg': {},
        'opus': {},
        'wav': {},
        'wma': {},
    }
    SUPPORTED_IN_VIDEO_FORMATS = {
        '3gp': {},
        'asf': {},
        'avi': {},
        'flv': {},
        'h264': {},
        'hevc': {},
        'm4v': {},
        'mkv': {},
        'mov': {},
        'mp4': {},
        'mpeg': {},
        'mpg': {},
        'webm': {},
        'wmv': {},
    }
    SUPPORTED_IN_FORMATS = SUPPORTED_IN_AUDIO_FORMATS | SUPPORTED_IN_VIDEO_FORMATS

    SUPPORTED_OUT_AUDIO_FORMATS = AudioFormatContainer.get_registered()
    SUPPORTED_OUT_VIDEO_FORMATS = VideoFormatContainer.get_registered()
    SUPPORTED_OUT_FORMATS = SUPPORTED_OUT_VIDEO_FORMATS | SUPPORTED_OUT_AUDIO_FORMATS

    @classmethod
    def __get_supported_codecs(cls, supported_format: dict[str, tuple[tuple, dict[str, Any]]], is_audio: bool, ext: str | None = None) -> Iterable[str]:
        res: set[str] = set()
        codecs_kwarg = "available_audio_codecs" if is_audio else "available_video_codecs"
        for cont_ext, data in supported_format.items():
            if not ext or (ext == cont_ext):
                _, kwargs = data
                res.update(kwargs[codecs_kwarg])
        return sorted(res)

    @classmethod
    def get_supported_audio_codecs(cls, ext: str | None = None) -> Iterable[str]:
        return cls.__get_supported_codecs(cls.SUPPORTED_OUT_FORMATS, is_audio=True, ext=ext)

    @classmethod
    def get_supported_video_codecs(cls, ext: str | None = None) -> Iterable[str]:
        return cls.__get_supported_codecs(cls.SUPPORTED_OUT_FORMATS, is_audio=False, ext=ext)

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "ffmpeg": "ffmpeg"
                }),
                BrewPackageManager({
                    "ffmpeg": "ffmpeg"
                }),
            },
            install_answer=install_deps,
        )
        self._install_deps = install_deps
        self._verbose = verbose

        # check ffmpeg
        self._ffmpeg_bin = self.find_in_path("ffmpeg")
        self._ffprobe_bin = self.find_in_path("ffprobe")



# src\file_conversor\cli\text\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.text._typer import COMMAND_NAME

from file_conversor.cli.text.check_cmd import typer_cmd as check_cmd
from file_conversor.cli.text.compress_cmd import typer_cmd as compress_cmd
from file_conversor.cli.text.convert_cmd import typer_cmd as convert_cmd

_ = get_translation()

text_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Text file manipulation (json, xml, etc)"),
)
text_cmd.add_typer(check_cmd)
text_cmd.add_typer(compress_cmd)
text_cmd.add_typer(convert_cmd)



# src\file_conversor\cli\hash\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

COMMAND_NAME = "hash"

# SUBCOMMANDS
CHECK_NAME = "check"
CREATE_NAME = "create"



# src\file_conversor\utils\dominate_bulma\page_base.py

import dominate
import dominate.tags

from typing import cast

# user-provided modules
from file_conversor.utils.dominate_bulma.modal import ModalCard
from file_conversor.utils.dominate_bulma.navbar import Navbar
from file_conversor.utils.dominate_bulma.footer import Footer
from file_conversor.utils.dominate_bulma.status_bar import StatusBar

from file_conversor.utils.dominate_utils import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation, AVAILABLE_LANGUAGES

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageBase(
        *content,
        nav_title,
        _title: str,
        **kwargs
):
    """
    Create the base page structure.

    :param _title: The title of the page.
    :param nav_title: The title or breadcrumb component for the navbar.
    :param content: The main content of the page.
    """
    with document(
        title=_title,
        _lang="en",
        _class="has-navbar-fixed-top has-status-bar-fixed-bottom",
        **kwargs,
    ) as component:
        with cast(dominate.tags.head, component.head) as head:
            meta(_charset="UTF-8")
            meta(_name="viewport", _content="width=device-width, initial-scale=1.0")
            # title(_title)

            # Favicon
            link(_rel="icon", _type="image/x-icon", _href="/icons/icon.png")
            link(_rel="icon", _type="image/png", _href="/icons/icon.png")

            # Libraries CSS
            link(_rel="stylesheet", _href="/static/css/bulma.min.css")
            link(_rel="stylesheet", _href="/static/css/font-awesome.all.min.css")

            # Custom CSS
            link(_rel="stylesheet", _href="/static/css/base.css")
        with cast(dominate.tags.body, component.body) as body:
            body.set_attribute("style", f"zoom: {CONFIG['gui-zoom']}%;")

            body.set_attribute("x-data", f"""{{ }}""")
            body.set_attribute(":class", f"""{{
                'is-cursor-wait': $store.status_bar.started && ! $store.status_bar.finished,
            }}""")

            # modal
            ModalCard(
                _class_head="p-5",
                _class_body="p-5",
                _class_foot="p-4",
            )

            # navbar
            Navbar(title=nav_title)

            with div(_id="main-app", _class="container") as main_app:
                main_app.add(*content)

            # footer
            Footer()
            StatusBar()

            # scripts
            script(_src="/static/js/base.js", _type="module", _defer=True)

            # alpine
            script(_src="/static/js/libs/alpine.mask.min.js", _defer=True)
            script(_src="/static/js/libs/alpine.sort.min.js", _defer=True)
            script(_src="/static/js/libs/alpine.min.js", _defer=True)
    return component


__all__ = [
    'PageBase'
]




# src/file_conversor/utils/__init__.py

"""
This module initializes the utils package.
It can contain utility functions or classes that are used across the application.
"""

from file_conversor.utils.abstract_register_manager import AbstractRegisterManager
from file_conversor.utils.command_manager import CommandManager
from file_conversor.utils.progress_manager import ProgressManager

from file_conversor.utils.rich_utils import (get_progress_bar,
                                             )



# src\file_conversor\backend\gui\_components\card.py

from typing import Any

from file_conversor.utils.dominate_utils import *


def Card(
    content: Any = None,
        top_image: Any = None,
        header_data: dict[str, Any] | None = None,
        footer_data: list[dict[str, Any]] | None = None,
        **kwargs,
):
    """
    Create a card component.

    :param content: The card content data to display in the card.
    :param top_image: The card image content (Figure or img tag).
    :param header_data: The card header content. {"title": str, "icon": FontAwesomeIcon}
    :param footer_data: The card footer content. list[{"text": str, "link": str}]
    """
    with div(_class="card", **kwargs) as card:
        # Card image section
        if top_image:
            with div(_class="card-image") as card_image:
                card_image.add(top_image)

        if header_data:
            with header(_class="card-header"):
                p(header_data["title"], _class="card-header-title")
                button(header_data["icon"], _class="card-header-icon", _aria_label="more options")

        # Card content section
        if content:
            with div(_class="card-content") as card_content:
                card_content.add(content)

        # Card footer section (optional)
        if footer_data:
            with footer(_class="card-footer") as card_footer:
                for item in footer_data:
                    a(item["text"], _class="card-footer-item", _href=item["link"])
    return card


__all__ = [
    'Card',
]



# src\file_conversor\config\config.py

import json
import locale

from pathlib import Path
from typing import Any

from file_conversor.config.environment import Environment


class Configuration:

    __instance = None

    @classmethod
    def get_instance(cls):
        if not cls.__instance:
            cls.__instance = cls()
        return cls.__instance

    def __init__(self) -> None:
        super().__init__()

        self.__config_path = Environment.get_resources_folder() / ".config.json"
        # Define configuration dictionary
        language = "en_US"
        if locale.getlocale() and locale.getlocale()[0]:
            language = locale.getlocale()[0]
        self.__data = {
            "port": 5000,              # Default port (flask app)
            "language": language,      # Default: system language or "en_US"
            "install-deps": True,      # Default: ask user to confirm dependency installation
            "audio-bitrate": 0,        # Default audio bitrate in kbps
            "video-bitrate": 0,        # Default video bitrate in kbps
            "video-format": "mp4",     # Default video format
            "video-encoding-speed": "medium",  # Default video encoding speed
            "video-quality": "medium",  # Default video quality
            "image-quality": 90,        # Default image quality 90%
            "image-dpi": 200,           # Default image => PDF dpi
            "image-fit": 'into',        # Default image => PDF fit mode
            "image-page-size": None,    # Default image => PDF page size
            "image-resampling": "bicubic",  # Default image resampling algorithm
            "pdf-compression": "medium",  # Default PDF compression level
            "gui-zoom": 100,            # Default GUI zoom level
            "gui-output-dir": str(Environment.UserFolder.DOWNLOADS()),  # Default output directory
        }

        self.load()

    def __repr__(self) -> str:
        return repr(self.__data)

    def __str__(self) -> str:
        return str(self.__data)

    def __getitem__(self, key) -> Any:
        return self.__data[key]

    def __setitem__(self, key, value):
        self.__data[key] = value

    def __delitem__(self, key):
        del self.__data[key]

    def __contains__(self, key):
        return key in self.__data

    def __len__(self):
        return len(self.__data)

    def get_path(self) -> Path:
        return self.__config_path

    def to_dict(self):
        return self.__data.copy()

    def clear(self):
        self.__data.clear()
        self.load()

    def update(self, new: dict):
        self.__data.update(new)

    def load(self):
        """Load app configuration file"""
        if self.__config_path.exists():
            self.__data.update(json.loads(self.__config_path.read_text()))

    def save(self):
        """Save app configuration file"""
        self.__config_path.write_text(json.dumps(self.__data, indent=2))




# src\file_conversor\cli\pdf\rotate_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.cli.pdf._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, ROTATE_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption, PasswordOption
from file_conversor.utils.formatters import parse_pdf_rotation

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PyPDFBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in PyPDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="rotate_anticlock_90",
                description="Rotate Left",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{ROTATE_NAME}" "%1" -r "1-:-90"',
                icon=str(icons_folder_path / "rotate_left.ico"),
            ),
            WinContextCommand(
                name="rotate_clock_90",
                description="Rotate Right",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{ROTATE_NAME}" "%1" -r "1-:90"',
                icon=str(icons_folder_path / "rotate_right.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_rotate_cmd(
    input_files: List[Path],
    rotation: List[str],
    password: str | None,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pypdf_backend = PyPDFBackend(verbose=STATE["verbose"])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        pypdf_backend.rotate(
            input_file=input_file,
            output_file=output_file,
            decrypt_password=password,
            rotations=parse_pdf_rotation(rotation, pypdf_backend.len(input_file)),
            progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p)),
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_rotated")
    logger.info(f"{_('Rotate pages')}: [bold green]{_('SUCCESS')}[/].")


@typer_cmd.command(
    name=ROTATE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Rotate PDF pages (clockwise or anti-clockwise).')}
        
        {_('Outputs a file with _rotated at the end.')}
    """,
    epilog=f"""
**{_('Examples')}:** 



*{_('Rotate page 1 by 180 degress')}*:

- `file_conversor {COMMAND_NAME} {ROTATE_NAME} input_file.pdf -o output_file.pdf -r "1:180"` 



*{_('Rotate page 5-7 by 90 degress, 9 by -90 degrees, 10-15 by 180 degrees')}*:

- `file_conversor {COMMAND_NAME} {ROTATE_NAME} input_file.pdf -r "5-7:90" -r "9:-90" -r "10-15:180"`
    """)
def rotate(
    input_files: Annotated[List[Path], InputFilesArgument(PyPDFBackend)],
    rotation: Annotated[List[str], typer.Option("--rotation", "-r",
                                                help=_("List of pages to rotate. Format ``\"page:rotation\"`` or ``\"start-end:rotation\"`` or ``\"start-:rotation\"`` ..."),
                                                )],
    password: Annotated[str | None, PasswordOption()] = None,
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_rotate_cmd(
        input_files=input_files,
        rotation=rotation,
        password=password,
        output_dir=output_dir,
    )




# src\file_conversor\cli\ebook\__init__.py


import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.ebook._typer import COMMAND_NAME

from file_conversor.cli.ebook.convert_cmd import typer_cmd as convert_cmd

_ = get_translation()

ebook_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Ebook file manipulation (requires Calibre external library)"),
)

# CONVERSION_PANEL
ebook_cmd.add_typer(convert_cmd)



# src\file_conversor\backend\gui\_components\icon.py

from file_conversor.utils.dominate_utils import *


def FontAwesomeIcon(
        name: str,
        _class: str = "",
        **kwargs,
):
    """
    Create a FontAwesome icon component.

    :param name: The name of the icon (e.g., "home", "settings").
    :param _class: Additional CSS classes to apply to the icon.
    """
    with span(_class=f"icon {_class}", **kwargs) as component:
        i(_class=f"fas fa-{name}")
    return component


__all__ = ["FontAwesomeIcon"]



# src\file_conversor\backend\ghostscript_backend.py

"""
This module provides functionalities for handling files using ``ghostscript`` backend.
"""

import re

from pathlib import Path

from enum import Enum
from typing import Any, Callable

# user-provided imports
from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.dependency import ScoopPackageManager, BrewPackageManager

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class GhostscriptBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``ghostscript``.
    """
    class OutputFileFormat(Enum):
        PDF = "pdfwrite"

        @classmethod
        def get_dict(cls):
            return {
                "pdf": cls.PDF,
            }

        @classmethod
        def from_str(cls, name: str):
            return cls.get_dict()[name]

        def get_options(self) -> list[str]:
            return [f"-sDEVICE={self.value}"]

    class Compression(Enum):
        HIGH = "screen"
        """72 dpi quality - high compression / low quality"""
        MEDIUM = "ebook"
        """150 dpi quality - medium compression / medium quality"""
        LOW = "printer"
        """300 dpi quality - low compression / high quality"""
        NONE = "preprint"
        """600 dpi quality - no compression / highest quality"""

        @classmethod
        def get_dict(cls):
            return {
                "high": cls.HIGH,
                "medium": cls.MEDIUM,
                "low": cls.LOW,
                "none": cls.NONE,
            }

        @classmethod
        def from_str(cls, name: str):
            return cls.get_dict()[name]

        def get_options(self) -> list[str]:
            return [f"-dPDFSETTINGS=/{self.value}"]

    class CompatibilityPreset(Enum):
        PRESET_1_3 = "1.3"
        """legacy option"""
        PRESET_1_4 = "1.4"
        """legacy option"""
        PRESET_1_5 = "1.5"
        """good campatibility / support for object stream compression"""
        PRESET_1_6 = "1.6"
        """medium campatibility / support for JPEG2000 compression"""
        PRESET_1_7 = "1.7"
        """low campatibility / support for 3D and transparency"""

        @classmethod
        def get_dict(cls):
            return {
                "1.3": cls.PRESET_1_3,
                "1.4": cls.PRESET_1_4,
                "1.5": cls.PRESET_1_5,
                "1.6": cls.PRESET_1_6,
                "1.7": cls.PRESET_1_7,
            }

        @classmethod
        def from_str(cls, name: str):
            return cls.get_dict()[name]

        def get_options(self) -> list[str]:
            return [f"-dCompatibilityLevel={self.value}"]  # PDF preset

    class Downsampling(Enum):
        HIGH = "Bicubic"
        """slowest processing / highest quality"""
        MEDIUM = "Average"
        """medium processing / medium quality"""
        LOW = "Subsample"
        """fast processing / low quality"""

        @classmethod
        def get_dict(cls):
            return {
                "high": cls.HIGH,
                "medium": cls.MEDIUM,
                "low": cls.LOW,
            }

        @classmethod
        def from_str(cls, name: str):
            return cls.get_dict()[name]

        def get_options(self) -> list[str]:
            return [
                f"-dDownsampleColorImages=true",
                f"-dDownsampleGrayImages=true",
                f"-dColorImageDownsampleType=/{self.value}",
                f"-dGrayImageDownsampleType=/{self.value}",
            ]

    class ImageCompression(Enum):
        JPX = "JPXEncode"
        """JPEG2000 format (poor support by browsers / open source viewers)"""
        JPG = "DCTEncode"
        """JPEG format (great support by browsers / open source viewers)"""
        PNG = "FlateEncode"
        """PNG format (great support / high file size)"""

        @classmethod
        def get_dict(cls):
            return {
                "jpx": cls.JPX,
                "jpg": cls.JPG,
                "png": cls.PNG,
            }

        @classmethod
        def from_str(cls, name: str):
            return cls.get_dict()[name]

        def get_options(self) -> list[str]:
            return [
                f"-dAutoFilterColorImages=false",
                f"-dAutoFilterGrayImages=false",
                f"-dColorImageFilter=/{self.value}",
                f"-dGrayImageFilter=/{self.value}",
            ]

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {
            "out_file_format": OutputFileFormat.PDF,
        },
    }
    EXTERNAL_DEPENDENCIES = {
        "gs",
    }

    PROGRESS_RE = re.compile(r'Page\s*(\d+)')
    NUM_PAGES_RE = re.compile(r'Processing pages (\d+) through (\d+)')

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 

        :raises RuntimeError: if dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "gs": "ghostscript"
                }),
                BrewPackageManager({
                    "gs": "ghostscript"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        # find ghostscript bin
        self._ghostscript_bin = self.find_in_path("gs")

    def compress(self,
                 output_file: str | Path,
                 input_file: str | Path,
                 compression_level: Compression,
                 compatibility_preset: CompatibilityPreset = CompatibilityPreset.PRESET_1_5,
                 downsampling_type: Downsampling = Downsampling.HIGH,
                 image_compression: ImageCompression = ImageCompression.JPG,
                 progress_callback: Callable[[float], Any] | None = None,
                 ):
        """
        Compress input PDF files.

        :param output_file: Output file
        :param input_file: Input file.         
        :param compression_level: Compression level.
        :param compatibility_level: PDF compatibility level (1.3 - 1.7). Defaults to ``CompatibilityPreset.PRESET_1_5`` (good compatibility / stream compression support).
        :param downsampling_type: Image downsampling type. Defaults to ``Downsampling.HIGH`` (best image quality / slower processing).
        :param image_compression: Image compression format. Defaults to `ImageCompression.JPG` (great compatibility / good compression).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if output format is unsupported
        """
        self.check_file_exists(input_file)
        in_path = Path(input_file)
        out_path = Path(output_file)

        out_ext = out_path.suffix[1:]
        if out_ext not in self.SUPPORTED_OUT_FORMATS:
            raise ValueError(f"Output format '{out_ext}' not supported")

        out_file_format: GhostscriptBackend.OutputFileFormat = self.SUPPORTED_OUT_FORMATS[out_ext]["out_file_format"]

        # build command
        command = [
            f"{self._ghostscript_bin}",

            # set non-interactive mode
            f"-dNOPAUSE",
            f"-dBATCH",
        ]

        command.extend(out_file_format.get_options())  # file_device options
        command.extend(compression_level.get_options())  # compression options
        command.extend(compatibility_preset.get_options())  # compatibility options
        command.extend(downsampling_type.get_options())  # downsampling options
        command.extend(image_compression.get_options())  # set image compression
        if image_compression == GhostscriptBackend.ImageCompression.JPG:
            if compression_level == GhostscriptBackend.Compression.HIGH:
                command.append(f"-dJPEGQ=70")  # Set JPEG quality
            elif compression_level == GhostscriptBackend.Compression.MEDIUM:
                command.append(f"-dJPEGQ=80")  # Set JPEG quality
            elif compression_level == GhostscriptBackend.Compression.LOW:
                command.append(f"-dJPEGQ=90")  # Set JPEG quality
            else:
                command.append(f"-dJPEGQ=99")  # Set JPEG quality

        # set input/output files
        command.extend([
            f"-sOutputFile={out_path}",
            f"{in_path}",
        ])

        # Execute the FFmpeg command
        process = Environment.run_nowait(
            *command,
        )

        out_lines: list[str] = []
        num_pages = 0
        while process.poll() is None:
            if not process.stdout:
                continue
            line = process.stdout.readline()

            match = GhostscriptBackend.NUM_PAGES_RE.search(line)
            if match:
                begin = int(match.group(1))
                end = int(match.group(2))
                num_pages = end - begin + 1

            match = GhostscriptBackend.PROGRESS_RE.search(line)
            if not match:
                out_lines.append(line)
                continue
            pages = int(match.group(1))
            progress = 100.0 * (float(pages) / num_pages)
            if progress_callback:
                progress_callback(progress)

        Environment.check_returncode(process, out_lines=out_lines)
        return process



# src/file_conversor/backend/gui/video/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.video._dom_page import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def video_index():
    tools = [
        {
            'image': url_for('icons', filename='convert.ico'),
            'title': _("Convert"),
            'subtitle': _("Convert a video file to another video format."),
            'url': url_for('video_convert'),
        },
        {
            'image': url_for('icons', filename='compress.ico'),
            'title': _("Compress"),
            'subtitle': _("Compress a video file to a target file size."),
            'url': url_for('video_compress'),
        },
        {
            'image': url_for('icons', filename='color.ico'),
            'title': _("Enhance"),
            'subtitle': _("Enhance video bitrate, resolution, fps, color, brightness, etc."),
            'url': url_for('video_enhance'),
        },
        {
            'image': url_for('icons', filename='left_right.ico'),
            'title': _("Mirror / Flip"),
            'subtitle': _("Mirror a video file (vertically or horizontally)."),
            'url': url_for('video_mirror'),
        },
        {
            'image': url_for('icons', filename='rotate_right.ico'),
            'title': _("Rotate"),
            'subtitle': _("Rotate a video file (clockwise or anti-clockwise)."),
            'url': url_for('video_rotate'),
        },
        {
            'image': url_for('icons', filename='resize.ico'),
            'title': _("Resize"),
            'subtitle': _("Resize video resolution (downscaling / upscaling)."),
            'url': url_for('video_resize'),
        },
        {
            'image': url_for('icons', filename='check.ico'),
            'title': _("Check"),
            'subtitle': _("Checks a audio/video file for corruption / inconsistencies."),
            'url': url_for('video_check'),
        },
        {
            'image': url_for('icons', filename='info.ico'),
            'title': _("Get info"),
            'subtitle': _("Get information about a audio/video file."),
            'url': url_for('video_info'),
        },
    ]
    return render_template_string(str(
        PageCardGrid(
            *tools,
            nav_items=[
                home_nav_item(),
                video_index_nav_item(active=True),
            ],
            _title=f"{_('Video Tools')} - File Conversor",
        )
    ))



# src\file_conversor\utils\bulma\tabs.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon

from file_conversor.utils.dominate_utils import *


class TabsAlignment:
    LEFT = "is-left"
    CENTER = "is-centered"
    RIGHT = "is-right"


class TabsSize:
    SMALL = "is-small"
    MEDIUM = "is-medium"
    LARGE = "is-large"


class TabsStyle:
    DEFAULT = ""
    BOXED = "is-boxed"
    TOGGLED = "is-toggle"
    TOGGLED_ROUNDED = "is-toggle is-toggle-rounded"


def Tabs(
    *tabs: dict[str, Any],
    active_tab: str = "",
    _class: str = "",
    _class_headers: str = "",
    _class_content: str = "",
    **kwargs,
):
    """
    Create a Bulma tabs component.

    :param tabs: A list of dictionaries representing each tab. Each dictionary should have:
                 - 'label': The text label of the tab.
                 - 'icon': (Optional) The FontAwesome icon name for the tab.
                 - 'content': The list of content to display when the tab is active.
    :param active_tab: The label of the tab that should be active by default.                 
    :param _class: Additional CSS classes for the tabs container.
    :param _class_headers: Additional CSS classes for the tab headers.
    :param _class_content: Additional CSS classes for the tab content area.
    """
    if not tabs:
        raise ValueError("At least one tab must be provided.")

    active_tab = active_tab or tabs[0]['label']
    if all(tab['label'] != active_tab for tab in tabs):
        raise ValueError("The active_tab must match one of the provided tab labels.")

    with div(_class=f"tabs {_class}", **{
        "x-data": f"""{{
            activeTab: '{active_tab}',
        }}""",
    }, **kwargs) as tabs_div:
        with ul(_class=f"tab-headers {_class_headers}"):
            for tab in tabs:
                with li(**{
                    ":class": f"""{{
                        'is-active': activeTab === '{tab['label']}'
                    }}"""
                }):
                    with a(**{
                        "@click.prevent": f"activeTab = '{tab['label']}'",
                    }):
                        if "icon" in tab:
                            FontAwesomeIcon(tab["icon"], _class="is-small")
                        span(tab["label"])
        for tab in tabs:
            with div(_class=f"tab-content {_class_content}", **{
                ":class": f"""{{
                    'is-hidden': activeTab !== '{tab['label']}'
                }}"""
            }) as tab_content_div:
                tab_content_div.add(*tab['content'])
    return tabs_div


__all__ = [
    'Tabs',
    'TabsAlignment',
    'TabsSize',
    'TabsStyle',
]




# src\file_conversor\cli\pdf\merge_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.cli.pdf._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, MERGE_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputFileOption, PasswordOption
from file_conversor.utils.validators import check_path_exists

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PyPDFBackend.EXTERNAL_DEPENDENCIES


def execute_pdf_merge_cmd(
    input_files: List[Path],
    password: str | None,
    output_file: Path | None,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    output_file = output_file if output_file else Path() / CommandManager.get_output_file(input_files[0], stem="_merged")
    if not STATE["overwrite-output"]:
        check_path_exists(output_file, exists=False)

    pypdf_backend = PyPDFBackend(verbose=STATE["verbose"])
    with ProgressManager() as progress_mgr:
        print(f"Processing '{output_file}' ...")
        pypdf_backend.merge(
            # files
            input_files=input_files,
            output_file=output_file,
            password=password,
            progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p))
        )
        progress_callback(progress_mgr.complete_step())

    logger.info(f"{_('Merge pages')}: [bold green]{_('SUCCESS')}[/].")


@typer_cmd.command(
    name=MERGE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Merge (join) input PDFs into a single PDF file.')}
        
        {_('Outputs a file with _merged at the end.')}
    """,
    epilog=f"""
**{_('Examples')}:** 



*{_('Merge files "input_file1.pdf" and "input_file2.pdf" into "output_file.pdf"')}*:

- `file_conversor {COMMAND_NAME} {MERGE_NAME} "input_file1.pdf" "input_file2.pdf" -of output_file.pdf` 



*{_('Merge protected PDFs "input_file1.pdf" and "input_file2.pdf" with password "unlock_password"')}*:

- `file_conversor {COMMAND_NAME} {MERGE_NAME} "input_file1.pdf" "input_file2.pdf" -p "unlock_password" -of output_file.pdf` 
    """)
def merge(
    input_files: Annotated[List[Path], InputFilesArgument(PyPDFBackend)],
    password: Annotated[str | None, PasswordOption()] = None,
    output_file: Annotated[Path | None, OutputFileOption(PyPDFBackend)] = None,
):
    execute_pdf_merge_cmd(
        input_files=input_files,
        password=password,
        output_file=output_file,
    )



# src\file_conversor\backend\audio_video\ffprobe_backend.py

"""
This module provides functionalities for handling audio and video files using FFprobe.
"""

import json

from pathlib import Path
from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.backend.audio_video.abstract_ffmpeg_backend import AbstractFFmpegBackend

from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.validators import check_file_format

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class FFprobeBackend(AbstractFFmpegBackend):
    """
    FFprobeBackend is a class that provides an interface for handling audio and video files using FFmpeg.
    """

    EXTERNAL_DEPENDENCIES = set([
        "ffprobe",
    ])

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if dependency is not found
        """
        super().__init__(install_deps=install_deps, verbose=verbose)

    def get_resolution(self, file_path: str | Path):
        metadata = self.info(file_path)
        if "streams" in metadata:
            for stream in metadata["streams"]:
                stream: dict[str, str]
                stream_type = stream.get("codec_type", "unknown").lower()
                if stream_type != "video":
                    continue
                width = int(stream.get('width', '0'))
                height = int(stream.get('height', '0'))
                return width, height if width > 0 and height > 0 else None, None
        return None, None

    def get_duration(self, file_path: str | Path) -> float:
        """
        Calculate file total duration (in secs), using `ffprobe`.

        :return: Total duration in seconds. Returns 0 if duration cannot be determined.
        """
        process = Environment.run(
            f'{self._ffprobe_bin}',
            f'-v',
            f'error',
            f'-show_entries',
            f'format=duration', '-of',
            f'default=noprint_wrappers=1:nokey=1',
            f'{file_path}',
        )
        duration_str = process.stdout.strip()
        try:
            return float(duration_str if duration_str else "0")
        except ValueError:
            return 0.0

    def info(self, file_path: str | Path) -> dict:
        """
        Get file metadata in JSON format

        result = {
            streams: [],
            chapters: [],
            format: {},
        }

        stream = {
            index,
            codec_name,
            codec_long_name,
            codec_type: audio|video,
            sampling_rate,
            channels,
            channel_layout: stereo|mono,
        }

        format = {
            format_name,
            format_long_name,
            duration,
            size,
        }

        :return: JSON object
        """
        result = Environment.run(
            f"{self._ffprobe_bin}",
            "-v", "quiet",
            "-print_format", "json",
            "-show_format",
            "-show_streams",
            "-show_chapters",
            "-show_error",
            f"{file_path}",
        )
        return json.loads(result.stdout)




# src\file_conversor\cli\video\__init__.py


import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.video._typer import COMMAND_NAME

from file_conversor.cli.video.check_cmd import typer_cmd as check_cmd
from file_conversor.cli.video.compress_cmd import typer_cmd as compress_cmd
from file_conversor.cli.video.convert_cmd import typer_cmd as convert_cmd
from file_conversor.cli.video.enhance_cmd import typer_cmd as enhance_cmd
from file_conversor.cli.video.execute_cmd import typer_cmd as execute_cmd
from file_conversor.cli.video.info_cmd import typer_cmd as info_cmd
from file_conversor.cli.video.list_formats_cmd import typer_cmd as list_formats_cmd
from file_conversor.cli.video.mirror_cmd import typer_cmd as mirror_cmd
from file_conversor.cli.video.resize_cmd import typer_cmd as resize_cmd
from file_conversor.cli.video.rotate_cmd import typer_cmd as rotate_cmd

_ = get_translation()

video_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Video file manipulation (requires FFMpeg external library)"),
)

# TRANSFORMATION_PANEL
video_cmd.add_typer(convert_cmd)
video_cmd.add_typer(compress_cmd)
video_cmd.add_typer(enhance_cmd)
video_cmd.add_typer(mirror_cmd)
video_cmd.add_typer(rotate_cmd)
video_cmd.add_typer(resize_cmd)


# OTHERS_PANEL
video_cmd.add_typer(check_cmd)
video_cmd.add_typer(info_cmd)
video_cmd.add_typer(list_formats_cmd)
video_cmd.add_typer(execute_cmd)



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon

from file_conversor.utils.dominate_bulma.form_button import Button
from file_conversor.utils.dominate_bulma.form_field import FormField

from file_conversor.utils.dominate_utils import *


def Input(
    _type: str = "text",
    _class: str = "",
    **kwargs,
):
    """
    Create an input element.

    :param _type: The type of the input (e.g., text, password, email).
    :param _class: Additional CSS classes for the input.
    """
    return input_(
        _type=_type,
        _class=f"input {_class}",
        **kwargs,
    )


def FormFieldInput(
    help: str,
    _name: str,
    _type: str = "text",
    current_value: str = "",
    label_text: str = "",
    validation_expr: str = "true",
    button: dict[str, Any] | None = None,
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and an input element.

    :param _name: The name attribute for the select element.
    :param _type: The type of the input (e.g., text, password, email).
    :param current_value: The current value.
    :param help: Optional help text.
    :param label_text: The text for the label.
    :param validation_expr: The validation expression for the input.
    :param button: Optional button to include next to the input. {"text": "Click Me", "_class": "button-class", "icon": {"name": "search"}}.

    :return: The FormField element.
    """
    field = FormField(
        Input(
            _class="is-flex is-flex-grow-1",
            _name=_name,
            _type=_type,
            _title=help,
            **{
                ':class': """{
                    'is-danger': !isValid,
                    'is-success': isValid,
                }""",
                'x-model': 'value',
                **kwargs,
            }
        ),
        Button(
            FontAwesomeIcon(**button.get('icon', {})),
            **{
                k: v
                for k, v in button.items()
                if k not in ['icon']
            },
        ) if button else None,
        has_addons=True if button else False,
        _class_control="is-flex is-flex-direction-column is-flex-grow-1",
        current_value=current_value,
        validation_expr=validation_expr,
        label_text=label_text,
        help=help,
        x_data=x_data,
        x_init=x_init,
    )
    return field


__all__ = [
    "Input",
    "FormFieldInput",
]



# src\file_conversor\system\lin\utils.py

import os
import platform

# Import only on Linux to avoid ImportError on other OSes
if platform.system() == "Linux":
    pass  # dummy, do nothing
else:
    pass  # Placeholder so the name exists


def is_admin() -> bool:
    """True if app running with admin priviledges, False otherwise."""
    return os.geteuid() == 0  # type: ignore


def reload_user_path():
    """Reload user PATH in current process."""
    # dummy, not needed in Linux
    pass


def set_window_icon(
    window_title: str,
    icon_path: str | os.PathLike,
    cx: int = 0,
    cy: int = 0,
):
    """
    Set the icon of a window given its title.
    Note: Setting window icons is not natively supported on Linux.
    This function is a placeholder and does not perform any action.
    """
    # Linux does not support setting window icons in the same way as Windows.
    pass


__all__ = [
    "is_admin",
    "reload_user_path",
    "set_window_icon",
]



# src/file_conversor/backend/gui/image/enhance.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.image import PillowBackend

from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.image._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageImageEnhance():
    return PageForm(
        InputFilesField(
            *[f for f in PillowBackend.SUPPORTED_IN_FORMATS],
            description=_("Image files"),
        ),
        BrightnessField(),
        ColorField(),
        ContrastField(),
        SharpnessField(),
        OutputDirField(),
        api_endpoint=f"{url_for('api_image_enhance')}",
        nav_items=[
            home_nav_item(),
            image_index_nav_item(),
            image_enhance_nav_item(active=True),
        ],
        _title=f"{_('Image Enhance')} - File Conversor",
    )


def image_enhance():
    return render_template_string(str(
        PageImageEnhance()
    ))



# src\file_conversor\cli\__init__.py

import sys
import typer

from rich import print

from pathlib import Path
from typing import Annotated, Any

# user-provided imports
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import AVAILABLE_LANGUAGES, get_system_locale, get_translation

from file_conversor.cli._typer import COMMANDS_LIST, PYTHON_VERSION

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()

# Create a Typer CLI application
app_cmd = typer.Typer(
    rich_markup_mode="markdown",
    no_args_is_help=True,
    context_settings={
        "help_option_names": ["-h", "--help"],
    }
)


#####################
# REGISTER COMMANDS #
#####################

for cmd_obj in COMMANDS_LIST:
    app_cmd.add_typer(**cmd_obj)


#####################
#     APP PANEL
#####################


def version_callback(value: bool):
    if value:
        VERSION = Environment.get_version()
        print(f"File Conversor {VERSION}")
        print(f"Python {PYTHON_VERSION} ({sys.executable})")
        raise typer.Exit()


# Main callback, to process global options
@app_cmd.callback(
    help=f"""
        # File Conversor - CLI
    """,
    epilog=f"""
        {_('For more information, visit')} [http://www.github.com/andre-romano/file_conversor](http://www.github.com/andre-romano/file_conversor)
    """)
def main_callback(
        no_log: Annotated[bool, typer.Option(
            "--no-log", "-nl",
            help=_("Disable file logs"),
            is_flag=True,
        )] = False,
        no_progress: Annotated[bool, typer.Option(
            "--no-progress", "-np",
            help=f"{_('Disable progress bars')}",
            is_flag=True,
        )] = False,
        quiet: Annotated[bool, typer.Option(
            "--quiet", "-q",
            help=f"{_('Enable quiet mode (only display errors and progress bars)')}",
            is_flag=True,
        )] = False,
        verbose: Annotated[bool, typer.Option(
            "--verbose", "-v",
            help=_("Enable verbose mode"),
            is_flag=True,
        )] = False,
        debug: Annotated[bool, typer.Option(
            "--debug", "-d",
            help=_("Enable debug mode"),
            is_flag=True,
        )] = False,
        version: Annotated[bool, typer.Option(
            "--version", "-V",
            help=_("Display version"),
            callback=version_callback,
            is_flag=True,
        )] = False,
        overwrite_output: Annotated[bool, typer.Option(
            "--overwrite-output", "-oo",
            help=f"{_('Overwrite output files')}. Defaults to False (do not overwrite).",
            is_flag=True,
        )] = False,
):
    STATE.update({
        "no-log": no_log,
        "no-progress": no_progress,
        "quiet": quiet,
        "verbose": verbose,
        "debug": debug,
        "overwrite-output": overwrite_output,
    })
    logger.debug(f"Python {PYTHON_VERSION} ({sys.executable})")
    logger.debug(f"Command: {sys.argv}")
    # Environment.get_executable()
    logger.debug(f"Resources folder: {Environment.get_resources_folder()}")
    logger.debug(f"Working directory: {Path().resolve()}")
    logger.debug(f"Config file: {CONFIG.get_path()}")
    logger.debug(f"Available languages: {sorted(AVAILABLE_LANGUAGES)} ({len(AVAILABLE_LANGUAGES)} entries)")
    logger.debug(f"Language (config / sys): ({CONFIG['language']} / {get_system_locale()})")



# src\file_conversor\cli\pipeline\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "pipeline"

# SUBCOMMANDS
CREATE_NAME = "create"
EXECUTE_NAME = "execute"



# src/file_conversor/backend/gui/doc/convert.py

from typing import Any
from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.office import DOC_BACKEND

from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.doc._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageConvert():

    return PageForm(
        InputFilesField(
            *[f for f in DOC_BACKEND.SUPPORTED_IN_FORMATS],
            description=_("Document files")
        ),
        FileFormatField(*[
            (f, f.upper())
            for f in DOC_BACKEND.SUPPORTED_OUT_FORMATS
        ], current_value="pdf"),
        OutputDirField(),
        api_endpoint=f"{url_for('api_doc_convert')}",
        nav_items=[
            home_nav_item(),
            doc_index_nav_item(),
            doc_convert_nav_item(active=True),
        ],
        _title=f"{_('Convert Doc')} - File Conversor",
    )


def doc_convert():
    return render_template_string(str(
        PageConvert()
    ))



# src\file_conversor\backend\mozjpeg_backend.py

"""
This module provides functionalities for handling files using mozjpeg.
"""

import subprocess

from pathlib import Path

# user-provided imports
from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.validators import check_file_format

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager
from file_conversor.backend.abstract_backend import AbstractBackend

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class _MozJPEGBackend(AbstractBackend):  # pyright: ignore[reportUnusedClass]
    """
    Provides an interface for handling files using mozjpeg.
    """

    SUPPORTED_IN_FORMATS = {
        'jpeg': {},
        'jpg': {},
    }
    SUPPORTED_OUT_FORMATS = {
        'jpg': {},
    }
    EXTERNAL_DEPENDENCIES = set([
        "cjpeg",
    ])

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "cjpeg": "mozjpeg"
                }),
                BrewPackageManager({
                    "cjpeg": "mozjpeg"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        # check ffprobe / ffmpeg
        self._mozjpeg_bin = self.find_in_path("cjpeg")

    def compress(
        self,
            input_file: str | Path,
            output_file: str | Path,
            quality: int,
            **kwargs,
    ):
        """
        Execute the command to compress the input file.

        :param input_file: Input file path.
        :param output_file: Output file path.      
        :param quality: Output image quality.              
        :param kwargs: Optional arguments.

        :return: Subprocess.CompletedProcess object

        :raises RuntimeError: If backend encounters an error during execution.
        """
        # Execute the command
        process = Environment.run(
            f"{self._mozjpeg_bin}",
            f"-quality", f"{quality}",
            f"-progressive",
            f"-optimize",
            f"{input_file}",
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=False,
        )

        # Save the compressed output to file
        with open(output_file, "wb") as fp:
            fp.write(process.stdout)
        return process



# src/file_conversor/backend/gui/text/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.text._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def text_index():
    tools = [
        {
            'image': url_for('icons', filename='check.ico'),
            'title': _("Check files"),
            'subtitle': _("Checks a text file schema compliance (json, xml, yaml, etc)."),
            'url': url_for('text_check'),
        },
        {
            'image': url_for('icons', filename='compress.ico'),
            'title': _("Compress / Minify"),
            'subtitle': _("Compress / minify text file formats (json, xml, yaml, etc)."),
            'url': url_for('text_compress'),
        },
        {
            'image': url_for('icons', filename='convert.ico'),
            'title': _("Convert"),
            'subtitle': _("Converts text file formats (json, xml, yaml, etc)."),
            'url': url_for('text_convert'),
        },
    ]
    return render_template_string(str(
        PageCardGrid(
            *tools,
            nav_items=[
                home_nav_item(),
                text_index_nav_item(active=True),
            ],
            _title=f"{_('Text Tools')} - File Conversor",
        )
    ))



# src/file_conversor/backend/gui/config/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.config._dom_page import *

from file_conversor.backend.gui.config._tab_audio_video import TabConfigAudioVideo
from file_conversor.backend.gui.config._tab_general import TabConfigGeneral
from file_conversor.backend.gui.config._tab_image import TabConfigImage
from file_conversor.backend.gui.config._tab_network import TabConfigNetwork
from file_conversor.backend.gui.config._tab_pdf import TabConfigPDF

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

from file_conversor.utils.dominate_bulma import *

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageConfig():
    return PageForm(
        Tabs(
            {
                'label': _('General'),
                'icon': 'cog',
                'content': TabConfigGeneral(),
            },
            {
                'label': _('Network'),
                'icon': 'network-wired',
                'content': TabConfigNetwork(),
            },
            {
                'label': _('Audio & Video'),
                'icon': 'film',
                'content': TabConfigAudioVideo(),
            },
            {
                'label': _('Image'),
                'icon': 'image',
                'content': TabConfigImage(),
            },
            {
                'label': _('PDF'),
                'icon': 'file-pdf',
                'content': TabConfigPDF(),
            },
            active_tab=_('General'),
            _class="""
                is-toggle 
                is-toggle-rounded 
                is-flex 
                is-full-width 
                is-flex-direction-column 
                is-align-items-center
                mb-4
            """,
            _class_headers="mb-4",
            _class_content="is-full-width",
        ),
        api_endpoint=f"{url_for('api_config')}",
        nav_items=[
            home_nav_item(),
            config_index_nav_item(active=True),
        ],
        _title=_("Config - File Conversor"),
    )


def config_index():
    return render_template_string(str(
        PageConfig()
    ))



# src\file_conversor\utils\bulma_utils\pdf.py

from typing import Any

# user-provided modules
from file_conversor.backend.pdf import GhostscriptBackend, PikePDFBackend, PyPDFBackend, OcrMyPDFBackend

from file_conversor.utils.dominate_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.utils.formatters import format_file_types_webview

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_language_name, get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def PDFCompressionField():
    """Create a form field for PDF compression selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (k, k.upper())
                for k in GhostscriptBackend.Compression.get_dict()
            ],
            current_value=str(CONFIG["pdf-compression"]),
            _name="pdf-compression",
            help=_("Select the PDF compression level to apply. Higher compression may reduce quality."),
        ),
        label_text=_("PDF Compression"),
    )


def PDFPasswordField(
    _validation_expr: str = "value.length >= 0",
    _name: str = "password",
    _placeholder=_('Enter password (optional)'),
    help: str = _("Password used to encrypt/decrypt the PDF files."),
    label_text: str = _("Password"),
    **kwargs: Any,
):
    """Create a form field for PDF password input."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr=_validation_expr,
            _type="password",
            _name=_name,
            _placeholder=_placeholder,
            help=help,
            **kwargs,
        ),
        label_text=label_text,
    )


def PDFEncryptionAlgorithmField():
    """Create a form field for PDF encryption algorithm selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (k, k.upper())
                for k in PyPDFBackend.EncryptionAlgorithm.get_dict()
            ],
            current_value="AES-256",
            _name="pdf-encryption-algorithm",
            help=_("Select the PDF encryption algorithm to use. Stronger algorithms (e.g., AES-256) provide better security."),
        ),
        label_text=_("PDF Encryption Algorithm"),
    )


def PDFPagesField(
        _validation_expr: str = "value.length > 0",
        _name: str = "pages",
        help: str = _("PDF pages to extract (comma-separated list). Format 'start-end'."),
        label_text: str = _("Pages"),
        **kwargs: Any,
):
    """Create a form field for specifying PDF pages to extract."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr=_validation_expr,
            _type="text",
            _name=_name,
            help=help,
            **kwargs,
        ),
        label_text=label_text,
    )


def PDFLanguageField():
    """Create a form field for specifying OCR languages."""
    backend = OcrMyPDFBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE['verbose'],
    )
    languages = [
        (k, get_language_name(k))
        for k in backend.get_available_languages().union(backend.get_available_remote_languages())
    ]
    languages.sort(key=lambda x: x[1])  # sort by language name
    return FormFieldHorizontal(
        FormFieldSelect(
            *languages,
            current_value="eng",
            _name="pdf-language",
            help=_("Select the OCR language to use."),
        ),
        label_text=_("OCR Language"),
    )


__all__ = [
    "PDFCompressionField",
    "PDFPasswordField",
    "PDFEncryptionAlgorithmField",
    "PDFPagesField",
    "PDFLanguageField",
]



# src\file_conversor\backend\audio_video\codec.py

from enum import Enum
from pathlib import Path
from typing import Any, Callable, Iterable, Self

# user-provided imports
from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter

from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import AbstractRegisterManager

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class _FFmpegCodec(AbstractRegisterManager):
    @classmethod
    def get_available_codecs(cls) -> dict[str, Any]:
        return cls.get_registered()

    def __init__(
        self,
        invalid_prefix: str,
        prefix: str,
        name: str,
        options: dict[str, Any] | None = None,
    ) -> None:
        super().__init__()
        self._invalid_prefix = invalid_prefix
        self._prefix = prefix
        self._name = name
        self._options: dict[str, str | int | None] = {}
        self.update(options or {})

    # PROPERTIES
    @property
    def name(self):
        return self._name

    # DUNDER METHODS
    def __eq__(self, value: object) -> bool:
        if isinstance(value, _FFmpegCodec):
            return (self._name == value._name)
        return False

    def __hash__(self) -> int:
        return hash(self._name)

    def __repr__(self) -> str:
        return f"{self._name} ({' '.join(self.get_options())})"

    def __str__(self) -> str:
        return self._name

    # METHODS
    def update(self, options: dict[str, Any]):
        for opt, val in options.items():
            self.set(opt, val)

    def add(self, option: str, value: Any = None):
        if option in self._options:
            if value:
                self._options[option] = f"{self._options[option]},{value}"
        else:
            self.set(option, value)

    def set(self, option: str, value: Any = None):
        self._options[option] = value

    def unset(self, option: str):
        if option in self._options:
            del self._options[option]

    def set_bitrate(self, bitrate: int):
        raise NotImplementedError("not implemented")

    def set_filters(self, *filters: FFmpegFilter):
        raise NotImplementedError("not implemented")

    def get_options(self) -> list[str]:
        res = [self._prefix, self._name]
        if not self._name or self._name.lower() == "null":
            return [self._invalid_prefix]
        if self._name.lower() == "copy":
            return res
        for key, value in self._options.items():
            if value:
                res.extend([str(key), str(value)])
                continue
            res.extend([str(key)])
        return res


class FFmpegAudioCodec(_FFmpegCodec):
    _REGISTERED: dict[str, tuple[tuple, dict[str, Any]]] = {}

    def __init__(
        self,
        name: str,
        options: dict[str, Any] | None = None,
    ) -> None:
        super().__init__(invalid_prefix="-an", prefix="-c:a",
                         name=name,
                         options=options,
                         )

    def set_bitrate(self, bitrate: int):
        self.set("-b:a", f"{bitrate}k")

    def set_filters(self, *filters: FFmpegFilter):
        for filter in filters:
            self.add("-af", filter.get())


class FFmpegVideoCodec(_FFmpegCodec):
    _REGISTERED: dict[str, tuple[tuple, dict[str, Any]]] = {}

    def __init__(
        self,
        name: str,
        options: dict[str, Any] | None = None,
        encoding_speed_opts: dict[str, dict[str, Any]] | None = None,
        quality_setting_opts: dict[str, dict[str, Any]] | None = None,
        profile_opts: dict[str, dict[str, Any]] | None = None,
    ) -> None:
        super().__init__(invalid_prefix="-vn", prefix="-c:v",
                         name=name,
                         options=options,
                         )
        self._encoding_speed_opts = encoding_speed_opts or {}
        self._quality_setting_opts = quality_setting_opts or {}
        self._profile_opts = profile_opts or {}

    def set_profile(self, profile: str):
        if profile not in self._profile_opts:
            logger.warning(f"'{profile}' {_("profile not available for codec")} '{self._name}'")
            return
        self.update(self._profile_opts[profile])

    def set_encoding_speed(self, speed: str):
        if speed not in self._encoding_speed_opts:
            logger.warning(f"'{speed}' {_("speed not available for codec")} '{self._name}'")
            return
        self.update(self._encoding_speed_opts[speed])

    def set_quality_setting(self, quality: str):
        if quality not in self._quality_setting_opts:
            logger.warning(f"'{quality}' {_("quality not available for codec")} '{self._name}'")
            return
        self.update(self._quality_setting_opts[quality])
        self.unset("-b:v")  # remove bitrate if set

    def set_bitrate(self, bitrate: int):
        self.set("-b:v", f"{bitrate}k")

    def set_filters(self, *filters: FFmpegFilter):
        for filter in filters:
            self.add("-vf", filter.get())


# register AUDIO codecs
FFmpegAudioCodec.register("null", name="null")
FFmpegAudioCodec.register("copy", name="copy")
FFmpegAudioCodec.register("aac", name="aac")
FFmpegAudioCodec.register("ac3", name="ac3")
FFmpegAudioCodec.register("flac", name="flac")
FFmpegAudioCodec.register("libfdk_aac", name="libfdk_aac")
FFmpegAudioCodec.register("libmp3lame", name="libmp3lame")
FFmpegAudioCodec.register("libopus", name="libopus")
FFmpegAudioCodec.register("libvorbis", name="libvorbis")
FFmpegAudioCodec.register("pcm_s16le", name="pcm_s16le")


# register VIDEO codecs
class QUALITY_OPTIONS:
    def __init__(
        self,
        high: str,
        medium: str,
        low: str,
    ):
        super().__init__()
        self.high = high
        self.medium = medium
        self.low = low

    def _get_quality_lib(self) -> dict[str, dict[str, str]]:
        return {
            "high": {"-crf": self.high},
            "medium": {"-crf": self.medium},
            "low": {"-crf": self.low},
        }

    def _get_quality_nvenc(self) -> dict[str, dict[str, str]]:
        return {
            "high": {"-cq": self.high},
            "medium": {"-cq": self.medium},
            "low": {"-cq": self.low},
        }

    def _get_quality_vaapi(self) -> dict[str, dict[str, str]]:
        return {
            "high": {"-qp": self.high},
            "medium": {"-qp": self.medium},
            "low": {"-qp": self.low},
        }

    def _get_quality_qsv(self) -> dict[str, dict[str, str]]:
        return {
            "high": {"-global_quality": self.high},
            "medium": {"-global_quality": self.medium},
            "low": {"-global_quality": self.low},
        }

    def get_quality(self, impl: str) -> dict[str, dict[str, str]]:
        if impl == "lib":
            return self._get_quality_lib()
        elif impl == "nvenc":
            return self._get_quality_nvenc()
        elif impl == "vaapi":
            return self._get_quality_vaapi()
        elif impl == "qsv":
            return self._get_quality_qsv()
        raise ValueError(f"Unknown implementation: {impl}")


FFmpegVideoCodec.register("null", name="null")
FFmpegVideoCodec.register("copy", name="copy")

for codec_name, impl in [("h264_nvenc", "nvenc"), ("hevc_nvenc", "nvenc"),
                         ("h264_vaapi", "vaapi"), ("hevc_vaapi", "vaapi"),
                         ("h264_qsv", "qsv"), ("hevc_qsv", "qsv"),
                         ("libx264", "lib"), ("libx265", "lib"),
                         ]:
    FFmpegVideoCodec.register(codec_name, name=codec_name,
                              quality_setting_opts=QUALITY_OPTIONS(**{
                                  "high": "18",
                                  "medium": "23",
                                  "low": "28",
                              }).get_quality(impl),
                              encoding_speed_opts={
                                  "fast": {"-preset": "fast"},
                                  "medium": {"-preset": "medium"},
                                  "slow": {"-preset": "slow"},
                              },
                              profile_opts={
                                  "high": {"-profile:v": "high"},
                                  "medium": {"-profile:v": "main"},
                                  "low": {"-profile:v": "baseline"},
                              },
                              )


for codec_name, impl in [("vp8_vaapi", "vaapi"), ("vp9_vaapi", "vaapi"),
                         ("vp8_qsv", "qsv"), ("vp9_qsv", "qsv"),
                         ("libvpx", "lib"), ("libvpx-vp9", "lib"),
                         ]:
    FFmpegVideoCodec.register(codec_name, name=codec_name,
                              quality_setting_opts=QUALITY_OPTIONS(**{
                                  "high": "13",
                                  "medium": "35",
                                  "low": "55",
                              }).get_quality(impl),
                              )

for codec_name, impl in [("av1_nvenc", "nvenc"),
                         ("av1_vaapi", "vaapi"),
                         ("av1_qsv", "qsv"),
                         ("libaom-av1", "lib"),
                         ]:
    FFmpegVideoCodec.register(codec_name, name=codec_name,
                              quality_setting_opts=QUALITY_OPTIONS(**{
                                  "high": "19",
                                  "medium": "29",
                                  "low": "42",
                              }).get_quality(impl),
                              )

FFmpegVideoCodec.register("mpeg4", name="mpeg4")



# src/file_conversor/backend/gui/pdf/split.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.pdf._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PagePDFSplit():
    return PageForm(
        InputFilesField(
            *PyPDFBackend.SUPPORTED_IN_FORMATS,
            description=_("PDF files"),
        ),
        PDFPasswordField(),
        OutputDirField(),
        api_endpoint=f"{url_for('api_pdf_split')}",
        nav_items=[
            home_nav_item(),
            pdf_index_nav_item(),
            pdf_split_nav_item(active=True),
        ],
        _title=f"{_('Split PDF')} - File Conversor",
    )


def pdf_split():
    return render_template_string(str(
        PagePDFSplit()
    ))



# src/file_conversor/backend/gui/audio/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.audio._index import audio_index

from file_conversor.backend.gui.audio.check import audio_check
from file_conversor.backend.gui.audio.convert import audio_convert
from file_conversor.backend.gui.audio.info import audio_info


def routes():
    return [
        FlaskRoute(
            rule="/audio",
            handler=audio_index
        ),
        FlaskRoute(
            rule="/audio/check",
            handler=audio_check,
        ),
        FlaskRoute(
            rule="/audio/convert",
            handler=audio_convert,
        ),
        FlaskRoute(
            rule="/audio/info",
            handler=audio_info,
        ),
    ]


__all__ = ['routes']



# src\file_conversor\config\environment.py

import subprocess
import shutil
import sys
import platformdirs
import platform

from importlib.resources import files
from importlib.metadata import version

from pathlib import Path
from enum import Enum

# user provided imports
from file_conversor.config.log import Log

# Get app config
logger = Log.get_instance().getLogger(__name__)


class Environment:

    __instance = None

    class UserFolder(Enum):
        @classmethod
        def RUNTIME(cls) -> Path:
            return platformdirs.user_runtime_path().resolve()

        @classmethod
        def DATA(cls) -> Path:
            return platformdirs.user_data_path().resolve()

        @classmethod
        def CONFIG(cls) -> Path:
            return platformdirs.user_config_path().resolve()

        @classmethod
        def CACHE(cls) -> Path:
            return platformdirs.user_cache_path().resolve()

        @classmethod
        def LOG(cls) -> Path:
            return platformdirs.user_log_path().resolve()

        @classmethod
        def STATE(cls) -> Path:
            return platformdirs.user_state_path().resolve()

        @classmethod
        def MUSIC(cls) -> Path:
            return platformdirs.user_music_path().resolve()

        @classmethod
        def PICTURES(cls) -> Path:
            return platformdirs.user_pictures_path().resolve()

        @classmethod
        def VIDEOS(cls) -> Path:
            return platformdirs.user_videos_path().resolve()

        @classmethod
        def DOCUMENTS(cls) -> Path:
            return platformdirs.user_documents_path().resolve()

        @classmethod
        def DOWNLOADS(cls) -> Path:
            return platformdirs.user_downloads_path().resolve()

        @classmethod
        def DESKTOP(cls) -> Path:
            return platformdirs.user_desktop_path().resolve()

    class SiteFolder(Enum):
        @classmethod
        def RUNTIME(cls) -> Path:
            return platformdirs.site_runtime_path().resolve()

        @classmethod
        def DATA(cls) -> Path:
            return platformdirs.site_data_path().resolve()

        @classmethod
        def CONFIG(cls) -> Path:
            return platformdirs.site_config_path().resolve()

        @classmethod
        def CACHE(cls) -> Path:
            return platformdirs.site_cache_path().resolve()

    @classmethod
    def remove(cls, src: str | Path, globs: str = "*", remove_src: bool = True, no_exists_ok: bool = True):
        """
        Remove dir or file, using globs / wildcards

        :param src: Source folder/file to remove
        :param globs: Globs/wildcards to match files/folders to remove. Defaults to "*" (all files/folders).
        :param remove_src: If True, remove the source folder itself. Valid only if src is a directory. Defaults to True.
        :param no_exists_ok: Do not raise error if file/folder does not exist. Defaults to True.

        :raises FileNotFoundError: if file/folder does not exist and no_exists_ok is False
        """
        src_path = Path(src).resolve()
        if not src_path.exists():
            if no_exists_ok:
                return
            raise FileNotFoundError(f"Source '{src_path}' does not exist")

        if src_path.is_file():
            src_path.unlink()  # Remove single file
            return

        if src_path.is_dir():
            if remove_src:
                shutil.rmtree(src_path)
                return
            for path in src_path.glob(globs):
                logger.debug(f"Removing '{path}' ... ")
                if path.is_dir():
                    shutil.rmtree(path)
                else:
                    path.unlink()  # Remove single file

    @classmethod
    def copy(cls, src: Path | str, dst: Path | str, overwrite: bool = False):
        """Copy a file or folder."""
        src = Path(src).resolve()
        dst = Path(dst).resolve()
        dst.parent.mkdir(parents=True, exist_ok=True)
        if not src.exists():
            raise FileNotFoundError(f"Source '{src}' does not exist")
        if dst.exists():
            if not overwrite:
                raise FileExistsError(f"Destination '{dst}' already exists")
            if dst.is_dir():
                shutil.rmtree(dst)
            else:
                dst.unlink()
        if src.is_dir():
            shutil.copytree(src, dst)
        else:
            shutil.copy2(src, dst)

    @classmethod
    def move(cls, src: Path | str, dst: Path | str, overwrite: bool = False):
        """Move a file or folder."""
        src = Path(src).resolve()
        dst = Path(dst).resolve()
        if not src.exists():
            raise FileNotFoundError(f"Source '{src}' does not exist")
        if dst.exists():
            if not overwrite:
                raise FileExistsError(f"Destination '{dst}' already exists")
            cls.remove(dst, remove_src=True, no_exists_ok=True)
        shutil.move(str(src), str(dst))

    @classmethod
    def touch(cls, path: Path | str, mode: int = 0o644, exists_ok: bool = True):
        """Create an empty file."""
        path = Path(path).resolve()
        if path.exists() and not exists_ok:
            raise FileExistsError(f"File '{path}' already exists")
        path.parent.mkdir(parents=True, exist_ok=True)
        path.touch(mode=mode, exist_ok=exists_ok)

    @classmethod
    def get_system_platform(cls) -> str:
        """Get the current system platform."""
        return platform.system()

    @classmethod
    def get_executable(cls) -> str:
        """Get the executable path for this app's CLI."""
        res = ""

        exe = shutil.which(sys.argv[0]) if sys.argv else None
        if exe and not exe.endswith(".py"):
            res = rf'"{exe}"'
        else:
            python_exe = sys.executable
            main_py = Path(rf"{cls.get_resources_folder()}/__main__.py")
            res = rf'"{python_exe}" "{main_py}"'

        logger.debug(f"Executable cmd: {res}")
        return res

    @classmethod
    def get_resources_folder(cls) -> Path:
        """Get the absolute path of the included folders in pip."""
        res_path = Path(str(files("file_conversor"))).resolve()
        return res_path

    @classmethod
    def get_web_folder(cls) -> Path:
        web_path = cls.get_resources_folder() / ".web"
        # logger.debug(f"Web path: {web_path}")
        return web_path

    @classmethod
    def get_icons_folder(cls) -> Path:
        """Get the absolute path of the included folders in pip."""
        icons_path = cls.get_resources_folder() / ".icons"
        # logger.debug(f"Icons path: {icons_path}")
        return icons_path

    @classmethod
    def get_locales_folder(cls) -> Path:
        locales_path = cls.get_resources_folder() / ".locales"
        # logger.debug(f"Locales path: {locales_path}")
        return locales_path

    @classmethod
    def get_version(cls) -> str:
        """Get the current version of the app."""
        return version("file_conversor")

    @classmethod
    def get_app_icon(cls) -> Path:
        """Get the path to the app icon."""
        return cls.get_icons_folder() / "icon.ico"

    @classmethod
    def get_instance(cls):
        if not cls.__instance:
            cls.__instance = cls()
        return cls.__instance

    @classmethod
    def run_nowait(cls,
                   *cmd: str,
                   text: bool = True,
                   encoding: str | None = None,
                   env: dict | None = None,
                   cwd: str | Path | None = None,
                   stdout: int | None = subprocess.PIPE,
                   stderr: int | None = subprocess.STDOUT,
                   **kwargs,
                   ) -> subprocess.Popen:
        """
        Run a process within Python using a standardized API

        :param cmd: Command to run.
        :param text: Parse stdout/stderr as text. Defaults to True.
        :param encoding: Text encoding. Defaults to None (use system locale).
        :param env: Environment (variables, PATH, etc). Defaults to None (same as the current python process).
        :param cwd: Current working directory. Defaults to None (same as the current python process).
        :param stdint: Pass to stdin, or not. Defaults to ``None``.
        :param stdout: Capture stdout, or not. Defaults to ``subprocess.PIPE``.
        :param stderr: Capture stderr, or not. Defaults to ``subprocess.STDOUT``.
        """
        logger.debug(f"Starting process ...")
        logger.debug(f"{" ".join(cmd)}")

        process = subprocess.Popen(
            cmd,
            stdin=kwargs.get("stdin"),
            stdout=stdout,
            stderr=stderr,
            cwd=cwd,
            env=env,
            text=text,
            encoding=encoding,

            # options
            close_fds=kwargs.get("close_fds", True),
            shell=kwargs.get("shell", False),
        )
        return process

    @classmethod
    def run(cls,
            *cmd: str,
            text: bool = True,
            encoding: str | None = None,
            env: dict | None = None,
            cwd: str | Path | None = None,
            stdout: int | None = subprocess.PIPE,
            stderr: int | None = subprocess.STDOUT,
            **kwargs,
            ) -> subprocess.CompletedProcess:
        """
        Run a process within Python, and wait for it to finish.

        :param cmd: Command to run.
        :param text: Parse stdout/stderr as text. Defaults to True.
        :param encoding: Text encoding. Defaults to None (use system locale).
        :param env: Environment (variables, PATH, etc). Defaults to None (same as the current python process).
        :param cwd: Current working directory. Defaults to None (same as the current python process).
        :param stdout: Capture stdout, or not. Defaults to ``subprocess.PIPE``.
        :param stderr: Capture stderr, or not. Defaults to ``subprocess.STDOUT``.

        :raises subprocess.CalledProcessError: if command failed (needs `wait` to work)
        :raises Exception: if communicate() failed (needs `wait` to work)
        """
        process = cls.run_nowait(
            *cmd,
            text=text,
            encoding=encoding,
            env=env,
            cwd=cwd,
            stdout=stdout,
            stderr=stderr,
            **kwargs,
        )
        try:
            output, error = process.communicate()
        except Exception:
            if process.poll() is None:
                process.kill()
                process.wait()
            raise

        if process.returncode != 0:
            raise subprocess.CalledProcessError(
                returncode=process.returncode,
                cmd=process.args,
                output=output,
                stderr=error,
            )

        return subprocess.CompletedProcess(
            args=process.args,
            returncode=process.returncode,
            stdout=output,
            stderr=error,
        )

    @classmethod
    def check_returncode(
        cls,
        process: subprocess.Popen | subprocess.CompletedProcess,
        out_lines: list[str] | None = None,
        err_lines: list[str] | None = None,
    ):
        """Raises subprocess.CalledProcessError if process.returncode != 0"""
        if process.returncode != 0:
            stdout: list[str] = (out_lines or []) + (process.stdout.readlines() if process.stdout else [])
            stderr: list[str] = (err_lines or []) + (process.stderr.readlines() if process.stderr else [])
            raise subprocess.CalledProcessError(
                returncode=process.returncode,
                cmd=process.args,
                output="\n".join([line.strip() for line in stdout if line.strip() != ""]),
                stderr="\n".join([line.strip() for line in stderr if line.strip() != ""]),
            )

    def __init__(self) -> None:
        super().__init__()


__all__ = [
    "Environment",
]



# src/file_conversor/backend/gui/_api/text/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.text.check import api_text_check
from file_conversor.backend.gui._api.text.compress import api_text_compress
from file_conversor.backend.gui._api.text.convert import api_text_convert


def routes():
    return [
        FlaskRoute(
            rule="/api/text/check",
            handler=api_text_check,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/text/compress",
            handler=api_text_compress,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/text/convert",
            handler=api_text_convert,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']



# src\file_conversor\cli\gui\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.gui._typer import COMMAND_NAME

from file_conversor.cli.gui.start_cmd import typer_cmd as start_cmd

_ = get_translation()

gui_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Graphical User Interface commands"),
)

gui_cmd.add_typer(start_cmd)



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_bulma.form_field import FormField

from file_conversor.utils.dominate_utils import *


def TextArea(
    _class: str = "",
    **kwargs,
):
    """
    Create a textarea element.

    :param _class: Additional CSS classes for the textarea.
    """
    return textarea(
        _class=f"textarea {_class}",
        **kwargs,
    )


def FormFieldTextArea(
    _name: str,
    current_value: str,
    help: str,
    label_text: str = "",
    validation_expr: str = "",
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and an input element.

    :param _name: The name attribute for the select element.
    :param current_value: The current value.
    :param help: Optional help text.
    :param label_text: The text for the label.
    :param validation_expr: The validation expression for the input.
    :param x_data: Additional Alpine.js x-data properties.

    :return: The FormField element.
    """
    field = FormField(
        TextArea(
            _class="is-flex is-flex-grow-1",
            _name=_name,
            _title=help,
            **{
                ':class': """{
                    'is-danger': !isValid,
                    'is-success': isValid,
                }""",
                'x-model': 'value',
                **kwargs,
            }
        ),
        current_value=current_value,
        validation_expr=validation_expr,
        _class_control="is-flex is-flex-direction-column is-flex-grow-1",
        label_text=label_text,
        help=help,
        x_data=x_data,
        x_init=x_init,
    )
    return field


__all__ = [
    "TextArea",
    "FormFieldTextArea",
]




# src\file_conversor\cli\image\convert_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List
from rich import print

# user-provided modules
from file_conversor.backend.image import PillowBackend

from file_conversor.cli.image._typer import CONVERSION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption, QualityOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # Pillow commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_jpg",
                description="To JPG",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "jpg" -q 90',
                icon=str(icons_folder_path / 'jpg.ico'),
            ),
            WinContextCommand(
                name="to_png",
                description="To PNG",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "png" -q 90',
                icon=str(icons_folder_path / 'png.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_convert_cmd(
    input_files: List[Path],
    format: str,
    quality: int,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda progress: None,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pillow_backend.convert(
            input_file=input_file,
            output_file=output_file,
            quality=quality,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{format}")

    logger.info(f"{_('Image convertion')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=CONVERT_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Convert a image file to a different format.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.webp -f jpg --quality 85`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.bmp -f png -od D:/Downloads`
    """)
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(PillowBackend)],
    format: Annotated[str, FormatOption(PillowBackend)],
    quality: Annotated[int, QualityOption()] = CONFIG["image-quality"],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_image_convert_cmd(
        input_files=input_files,
        format=format,
        quality=quality,
        output_dir=output_dir,
    )



# src\file_conversor\backend\audio_video\ffmpeg_filter.py

from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.formatters import parse_ffmpeg_filter
from file_conversor.utils.validators import check_valid_options

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class FFmpegFilter:

    @classmethod
    def from_str(cls, filter: str):
        name, args, kwargs = parse_ffmpeg_filter(filter)
        return cls(name, *args, **kwargs)

    def __init__(self, name: str, *args: str, **options: str) -> None:
        super().__init__()
        self._name = name
        self._args = args
        self._options = options

    def __hash__(self) -> int:
        return hash(self._name) ^ hash(self._args) ^ hash(frozenset(self._options.items()))

    def __eq__(self, value: object) -> bool:
        if isinstance(value, FFmpegFilter):
            return (self._name == value._name) and (self._args == value._args) and (self._options == value._options)
        return False

    def __repr__(self) -> str:
        return self.get()

    def __str__(self) -> str:
        return f"{self._name}({", ".join(self._args)}{", ".join(f"{k}={v}" for k, v in self._options.items())})"

    def get(self) -> str:
        res = f"{self._name}" + ("=" if (self._args or self._options) else "")
        res += ":".join(self._args) + ("," if self._args and self._options else "")
        res += ":".join(f"{k}={v}" for k, v in self._options.items())
        return res


# BRIGHTNESS, CONTRAST, SATURATION, GAMMA
def FFmpegFilterEq(brightness: float = 1.0, contrast: float = 1.0, saturation: float = 1.0, gamma: float = 1.0) -> FFmpegFilter:
    return FFmpegFilter("eq", brightness=str(brightness - 1), contrast=str(contrast), saturation=str(saturation), gamma=str(gamma),)


# RESIZE
def FFmpegFilterScale(width: int | str, height: int | str, force_original_aspect_ratio: str | None = None) -> FFmpegFilter:
    options: dict[str, str] = {}
    if force_original_aspect_ratio:
        check_valid_options(force_original_aspect_ratio, {"increase", "decrease", "disable"})
        options["force_original_aspect_ratio"] = force_original_aspect_ratio
    return FFmpegFilter("scale", str(width), str(height), **options)


# ROTATE
def FFmpegFilterTranspose(direction: int) -> FFmpegFilter:
    check_valid_options(direction, {0, 1, 2, 3})
    return FFmpegFilter("transpose", str(direction))


# MIRROR
def FFmpegFilterHflip() -> FFmpegFilter:
    return FFmpegFilter("hflip")


def FFmpegFilterVflip() -> FFmpegFilter:
    return FFmpegFilter("vflip")


# FPS
def FFmpegFilterMInterpolate(fps: int, mi_mode: str = "mci", mc_mode: str = "aobmc", me_mode: str = "bidir", vsbmc: int = 1) -> FFmpegFilter:
    return FFmpegFilter("minterpolate", fps=str(fps), mi_mode=mi_mode, mc_mode=mc_mode, me_mode=me_mode, vsbmc=str(vsbmc))


# UNSHARP
def FFmpegFilterUnsharp(luma_msize_x: int = 5, luma_msize_y: int = 5, luma_amount: float = 1.0) -> FFmpegFilter:
    return FFmpegFilter("unsharp", luma_msize_x=str(luma_msize_x), luma_msize_y=str(luma_msize_y), luma_amount=str(luma_amount))


# DESHAKE
def FFmpegFilterDeshake() -> FFmpegFilter:
    return FFmpegFilter("deshake")




from typing import Any, Self


class AbstractRegisterManager:
    _REGISTERED: dict[str, tuple[tuple, dict[str, Any]]] = {}
    """{name: (args, kwargs)}"""

    @classmethod
    def get_registered(cls):
        return dict(cls._REGISTERED)

    @classmethod
    def is_registered(cls, n: str) -> bool:
        return n in cls._REGISTERED

    @classmethod
    def register(cls, n: str, *args, **kwargs):
        cls._REGISTERED[n] = (args, kwargs)

    @classmethod
    def from_str(cls, name: str) -> Self:
        if name not in cls._REGISTERED:
            raise ValueError(f"'{name}' not registered. Registered options: {', '.join(cls.get_registered())}")
        args, kwargs = cls._REGISTERED[name]
        return cls(*args, **kwargs)



# src\file_conversor\backend\office\abstract_libreoffice_backend.py

from pathlib import Path
from typing import Any, Callable

# user-provided imports
from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.dependency.brew_pkg_manager import BrewPackageManager
from file_conversor.dependency.scoop_pkg_manager import ScoopPackageManager

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class AbstractLibreofficeBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``libreoffice``.
    """
    EXTERNAL_DEPENDENCIES = {
        "soffice",
    }

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "soffice": "libreoffice"
                }, buckets=[
                    "extras",
                ], env=[
                    r"C:\Users\Andre\scoop\apps\libreoffice\current\LibreOffice\program"
                ]),
                BrewPackageManager({
                    "soffice": "libreoffice"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        self._libreoffice_bin = self.find_in_path("soffice")

    def convert(
        self,
        files: list[tuple[Path, Path]],
        file_processed_callback: Callable[[Path], Any] | None = None,
    ):
        """
        Convert input file into an output file.

        :param files: List of tuples containing input and output file paths.

        :raises FileNotFoundError: if input file not found.
        """
        for input_file, output_file in files:
            self.check_file_exists(input_file)

            input_path = input_file.resolve()
            output_path = output_file.resolve()

            output_dir = output_path.parent
            output_format = output_path.suffix.lstrip(".").lower()

            # Execute command
            Environment.run(
                str(self._libreoffice_bin),
                "--headless",
                "--convert-to",
                str(output_format),
                "--outdir",
                str(output_dir),
                str(input_path)
            )
            if file_processed_callback:
                file_processed_callback(input_path)



# src\file_conversor\backend\gui\video\_dom_page.py

from flask import url_for
from typing import Any

# user-provided modules
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def video_index_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Video"),
        'url': url_for('video_index'),
        'active': active,
    }


def video_check_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Check"),
        'url': url_for('video_check'),
        'active': active,
    }


def video_compress_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Compress"),
        'url': url_for('video_compress'),
        'active': active,
    }


def video_convert_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Convert"),
        'url': url_for('video_convert'),
        'active': active,
    }


def video_enhance_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Enhance"),
        'url': url_for('video_enhance'),
        'active': active,
    }


def video_info_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Info"),
        'url': url_for('video_info'),
        'active': active,
    }


def video_mirror_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Mirror"),
        'url': url_for('video_mirror'),
        'active': active,
    }


def video_resize_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Resize"),
        'url': url_for('video_resize'),
        'active': active,
    }


def video_rotate_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Rotate"),
        'url': url_for('video_rotate'),
        'active': active,
    }


__all__ = [
    'video_index_nav_item',
    'video_check_nav_item',
    'video_compress_nav_item',
    'video_convert_nav_item',
    'video_enhance_nav_item',
    'video_info_nav_item',
    'video_mirror_nav_item',
    'video_resize_nav_item',
    'video_rotate_nav_item',
]



# src/file_conversor/backend/gui/_api/video/enhance.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.cli.video._ffmpeg_cmd import ffmpeg_cli_cmd

from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.utils import CommandManager, ProgressManager

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle video enhancing."""
    logger.debug(f"Video enhance thread received: {params}")

    logger.info(f"[bold]{_('Enhancing video files')}[/]...")
    ffmpeg_cli_cmd(
        input_files=[Path(i) for i in params['input-files']],
        file_format=params['file-format'],

        audio_bitrate=int(params.get('audio-bitrate') or 0),
        video_bitrate=int(params.get('video-bitrate') or 0),

        video_encoding_speed=params.get('video-encoding-speed') or None,
        video_quality=params.get('video-quality') or None,

        resolution=params.get('resolution') or None,
        fps=params.get('fps') or None,

        brightness=params.get('brightness') or 1.0,
        contrast=params.get('contrast') or 1.0,
        color=params.get('color') or 1.0,
        gamma=params.get('gamma') or 1.0,

        deshake=params.get('deshake') or False,
        unsharp=params.get('unsharp') or False,

        out_stem="_enhanced",
        output_dir=Path(params.get('output-dir') or ""),
        progress_callback=lambda p, pm: status.set_progress(pm.update_progress(p)),
    )

    logger.debug(f"{status}")


def api_video_enhance():
    """API endpoint to enhance video files."""
    logger.info(f"[bold]{_('Video enhance requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)



# src\file_conversor\backend\gui\index.py

from typing import Any
from flask import render_template_string, send_from_directory, url_for

# user-provided modules
from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.utils.dominate_bulma import *
from file_conversor.utils.dominate_utils import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageIndex(
    *items: dict[str, str],
    nav_items: list[dict[str, Any]],
):
    return PageCardGrid(
        *items,
        nav_items=nav_items,
        _title=f"{_('Home')} - File Conversor",
    )


def index():
    return render_template_string(str(
        PageIndex(
            # OFFICE tools
            {
                'image': url_for('icons', filename='docx.ico'),
                'title': _("Document Tools"),
                'subtitle': f"{_('Document file manipulation')} {_('(requires MS Office / LibreOffice)')})",
                'url': url_for('doc_index'),
            },
            {
                'image': url_for('icons', filename='xls.ico'),
                'title': _("Spreadsheet Tools"),
                'subtitle': f"{_('Spreadsheet file manipulation')} {_('(requires MS Office / LibreOffice)')})",
                'url': url_for('xls_index'),
            },
            {
                'image': url_for('icons', filename='ppt.ico'),
                'title': _("Presentation Tools"),
                'subtitle': f"{_('Presentation file manipulation')} {_('(requires MS Office / LibreOffice)')})",
                'url': url_for('ppt_index'),
            },

            # OTHER FILES
            {
                'image': url_for('icons', filename='mp3.ico'),
                'title': _("Audio Tools"),
                'subtitle': _("Audio file manipulation (requires FFMpeg external library)"),
                'url': url_for('audio_index'),
            },
            {
                'image': url_for('icons', filename='mp4.ico'),
                'title': _("Video Tools"),
                'subtitle': _("Video file manipulation (requires FFMpeg external library)"),
                'url': url_for('video_index'),
            },
            {
                'image': url_for('icons', filename='jpg.ico'),
                'title': _("Image Tools"),
                'subtitle': _("Image file manipulation"),
                'url': url_for('image_index'),
            },
            {
                'image': url_for('icons', filename='pdf.ico'),
                'title': _("PDF Tools"),
                'subtitle': _("PDF file manipulation."),
                'url': url_for('pdf_index'),
            },
            {
                'image': url_for('icons', filename='epub.ico'),
                'title': _("Ebook Tools"),
                'subtitle': _("Ebook file manipulation (requires Calibre external library)"),
                'url': url_for('ebook_index'),
            },
            {
                'image': url_for('icons', filename='json.ico'),
                'title': _("Text Tools"),
                'subtitle': _("Text file manipulation"),
                'url': url_for('text_index'),
            },
            {
                'image': url_for('icons', filename='sha256.ico'),
                'title': _("Hash Tools"),
                'subtitle': _("Hash file manipulation"),
                'url': url_for('hash_index'),
            },
            {
                'image': url_for('icons', filename='config.ico'),
                'title': _("Configuration"),
                'subtitle': _("Configure application default options"),
                'url': url_for('config_index'),
            },
            nav_items=[
                {
                    'label': _("Home"),
                    'url': '/',
                    'active': True,
                },
            ],
        )
    ))


def routes():
    return [
        FlaskRoute(
            rule="/",
            handler=index
        ),
    ]


__all__ = ['routes', 'PageIndex']



# src\file_conversor\backend\gui\ppt\_dom_page.py

from flask import url_for
from typing import Any

# user-provided modules
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def ppt_index_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Presentation"),
        'url': url_for('ppt_index'),
        'active': active,
    }


def ppt_convert_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Convert"),
        'url': url_for('ppt_convert'),
        'active': active,
    }


__all__ = [
    'ppt_index_nav_item',
    'ppt_convert_nav_item',
]



# src/file_conversor/backend/gui/video/rotate.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.audio_video import FFmpegBackend

from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.video._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def TabGeneral() -> list | tuple:
    return [
        InputFilesField(
            *[f for f in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS],
            description=_("Video files"),
        ),
        FileFormatField(*[
            (q, q.upper())
            for q in filter(lambda x: x.lower() != 'null', FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)
        ], current_value='mp4'),
        RotationField(),
        OutputDirField(),
    ]


def TabAdvanced() -> list | tuple:
    return [
        AudioBitrateField(),
        VideoBitrateField(),

        VideoEncodingSpeedField(),
        VideoQualityField(),
    ]


def PageVideoRotate():
    return PageForm(
        Tabs(
            {
                'label': _('General'),
                'icon': 'cog',
                'content': TabGeneral(),
            },
            {
                'label': _('Advanced'),
                'icon': 'tools',
                'content': TabAdvanced(),
            },
            active_tab=_('General'),
            _class="""
                is-toggle 
                is-toggle-rounded 
                is-flex 
                is-full-width 
                is-flex-direction-column 
                is-align-items-center
                mb-4
            """,
            _class_headers="mb-4",
            _class_content="is-full-width",
        ),
        api_endpoint=f"{url_for('api_video_rotate')}",
        nav_items=[
            home_nav_item(),
            video_index_nav_item(),
            video_rotate_nav_item(active=True),
        ],
        _title=f"{_('Video Rotate')} - File Conversor",
    )


def video_rotate():
    return render_template_string(str(
        PageVideoRotate()
    ))



# src/file_conversor/backend/gui/audio/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.audio._dom_page import audio_index_nav_item

from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def audio_index():
    tools = [
        {
            'image': url_for('icons', filename='check.ico'),
            'title': _("Check files"),
            'subtitle': _("Checks a audio file for corruption / inconsistencies."),
            'url': url_for('audio_check'),
        },
        {
            'image': url_for('icons', filename='convert.ico'),
            'title': _("Convert files"),
            'subtitle': _("Convert a audio/video file to an audio format."),
            'url': url_for('audio_convert'),
        },
        {
            'image': url_for('icons', filename='info.ico'),
            'title': _("Get info"),
            'subtitle': _("Get information about a audio file."),
            'url': url_for('audio_info'),
        },
    ]

    return render_template_string(str(PageCardGrid(
        *tools,
        nav_items=[
            home_nav_item(),
            audio_index_nav_item(active=True),
        ],
        _title=f"{_('Audio Tools')} - File Conversor",
    )))



# src\file_conversor\backend\audio_video\__init__.py

from file_conversor.backend.audio_video.ffmpeg_backend import FFmpegBackend
from file_conversor.backend.audio_video.ffprobe_backend import FFprobeBackend



# src\file_conversor\backend\http_backend.py

"""
This module provides functionalities for handling repositories using HTTP.
"""

import requests

from pathlib import Path
from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.config import Environment, Log, get_translation

from file_conversor.utils.validators import check_file_format

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class HttpBackend(AbstractBackend):
    """
    HttpBackend is a class that provides an interface for handling HTTP requests.
    """

    SUPPORTED_IN_FORMATS = {}

    SUPPORTED_OUT_FORMATS = {}

    EXTERNAL_DEPENDENCIES = set()

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def get_json(
        self,
        url: str,
        **kwargs,
    ) -> Any:
        """Get JSON data from a URL.

        :param url: The URL to fetch the JSON data from.
        :param kwargs: Additional arguments to pass to the requests.get() method.
        :return: The JSON data.

        :raises RuntimeError: if the request fails or the response is not JSON.
        """
        response = requests.get(url, **kwargs)
        if not response.ok:
            raise RuntimeError(f"{_('Failed to get JSON from url')} '{url}': {response.status_code} - {response.text}")
        try:
            return response.json()
        except Exception as e:
            raise RuntimeError(f"{_('Failed to parse JSON from url')} '{url}': {e}")

    def download(
        self,
        url: str,
        dest_folder: str | Path,
        progress_callback: Callable[[float], Any] | None = None,
    ):
        """
        Download a file from a URL.
        """
        dest_path = Path(dest_folder).resolve()
        dest_path.parent.mkdir(parents=True, exist_ok=True)
        downloaded_bytes = 0.0

        with requests.get(url, stream=True) as response:
            if not response.ok:
                raise RuntimeError(f"{_('Failed to download file')}: {response.status_code} - {response.text}")
            total_size = float(response.headers.get("content-length", 0))
            with dest_path.open("wb") as f:
                for data in response.iter_content(chunk_size=1024):
                    f.write(data)
                    downloaded_bytes += len(data)
                    if progress_callback:
                        progress_callback(downloaded_bytes / total_size * 100)
        if not dest_path.exists():
            raise RuntimeError(f"{_('Failed to download file')}: '{dest_path}'")



# src\file_conversor\cli\vidio\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

TRANSFORMATION_PANEL = _("Transformations")
OTHERS_PANEL = _("Other commands")

# command
COMMAND_NAME = "video"

# SUBCOMMANDS
CHECK_NAME = "check"
COMPRESS_NAME = "compress"
CONVERT_NAME = "convert"
ENHANCE_NAME = "enhance"
EXECUTE_NAME = "execute"
INFO_NAME = "info"
LIST_FORMATS_NAME = "list-formats"
MIRROR_NAME = "mirror"
RESIZE_NAME = "resize"
ROTATE_NAME = "rotate"



# src/file_conversor/backend/gui/_api/pdf/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.pdf.compress import api_pdf_compress
from file_conversor.backend.gui._api.pdf.convert import api_pdf_convert
from file_conversor.backend.gui._api.pdf.decrypt import api_pdf_decrypt
from file_conversor.backend.gui._api.pdf.encrypt import api_pdf_encrypt
from file_conversor.backend.gui._api.pdf.extract_img import api_pdf_extract_img
from file_conversor.backend.gui._api.pdf.extract import api_pdf_extract
from file_conversor.backend.gui._api.pdf.merge import api_pdf_merge
from file_conversor.backend.gui._api.pdf.ocr import api_pdf_ocr
from file_conversor.backend.gui._api.pdf.repair import api_pdf_repair
from file_conversor.backend.gui._api.pdf.rotate import api_pdf_rotate
from file_conversor.backend.gui._api.pdf.split import api_pdf_split


def routes():
    return [
        FlaskRoute(
            rule="/api/pdf/compress",
            handler=api_pdf_compress,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/convert",
            handler=api_pdf_convert,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/decrypt",
            handler=api_pdf_decrypt,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/encrypt",
            handler=api_pdf_encrypt,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/extract_img",
            handler=api_pdf_extract_img,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/extract",
            handler=api_pdf_extract,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/merge",
            handler=api_pdf_merge,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/ocr",
            handler=api_pdf_ocr,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/repair",
            handler=api_pdf_repair,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/rotate",
            handler=api_pdf_rotate,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/pdf/split",
            handler=api_pdf_split,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']



# src/file_conversor/backend/gui/_api/video/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.video.check import api_video_check
from file_conversor.backend.gui._api.video.compress import api_video_compress
from file_conversor.backend.gui._api.video.convert import api_video_convert
from file_conversor.backend.gui._api.video.enhance import api_video_enhance
from file_conversor.backend.gui._api.video.info import api_video_info
from file_conversor.backend.gui._api.video.mirror import api_video_mirror
from file_conversor.backend.gui._api.video.resize import api_video_resize
from file_conversor.backend.gui._api.video.rotate import api_video_rotate


def routes():
    return [
        FlaskRoute(
            rule="/api/video/check",
            handler=api_video_check,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/compress",
            handler=api_video_compress,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/convert",
            handler=api_video_convert,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/enhance",
            handler=api_video_enhance,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/info",
            handler=api_video_info,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/mirror",
            handler=api_video_mirror,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/resize",
            handler=api_video_resize,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/video/rotate",
            handler=api_video_rotate,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']



# src\file_conversor\backend\pillow_backend.py

"""
This module provides functionalities for handling image files using ``pillow`` backend.
"""

from enum import Enum
from PIL import Image, ImageFilter, ImageEnhance, ImageOps
from PIL.ExifTags import TAGS

from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.formatters import normalize_degree

from file_conversor.backend.abstract_backend import AbstractBackend

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class PillowBackend(AbstractBackend):
    """
    A class that provides an interface for handling image files using ``pillow``.
    """
    class AntialiasAlgorithm(Enum):
        MEDIAN = ImageFilter.MedianFilter
        MODE = ImageFilter.ModeFilter

        @classmethod
        def from_str(cls, name: str):
            name = name.lower()
            return cls.get_dict()[name]

        @classmethod
        def get_dict(cls):
            return {
                "median": cls.MEDIAN,
                "mode": cls.MODE,
            }

    PILLOW_FILTERS: dict[str, type[ImageFilter.BuiltinFilter]] = {
        "blur": ImageFilter.BLUR,
        "smooth": ImageFilter.SMOOTH,
        "smooth_more": ImageFilter.SMOOTH_MORE,
        "sharpen": ImageFilter.DETAIL,
        "sharpen_more": ImageFilter.SHARPEN,
        "edge_enhance": ImageFilter.EDGE_ENHANCE,
        "edge_enhance_more": ImageFilter.EDGE_ENHANCE_MORE,
        "emboss": ImageFilter.EMBOSS,
        "emboss_edge": ImageFilter.CONTOUR,
    }

    Exif = Image.Exif
    Exif_TAGS = TAGS

    RESAMPLING_OPTIONS = {
        "bicubic": Image.Resampling.BICUBIC,
        "bilinear": Image.Resampling.BILINEAR,
        "lanczos": Image.Resampling.LANCZOS,
        "nearest": Image.Resampling.NEAREST,
    }

    SUPPORTED_IN_FORMATS = {
        "bmp": {},
        "gif": {},
        "ico": {},
        "jfif": {},
        "jpg": {},
        "jpeg": {},
        "jpe": {},
        "png": {},
        "psd": {},
        "tif": {},
        "tiff": {},
        "webp": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "bmp": {"format": "BMP"},
        "gif": {"format": "GIF"},
        "ico": {"format": "ICO"},
        "jpg": {"format": "JPEG"},
        "apng": {"format": "PNG"},
        "png": {"format": "PNG"},
        "pdf": {"format": "PDF"},
        "tif": {"format": "TIFF"},
        "webp": {"format": "WEBP"},
    }
    EXTERNAL_DEPENDENCIES = set([])

    def __init__(self, verbose: bool = False,):
        """
        Initialize the ``pillow`` backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def info(self, input_file: str | Path,) -> Exif:
        """
        Get EXIF info from input file.

        :param input_file: Input image file.

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        img = self._open(input_file)
        return img.getexif()

    def resize(self,
               output_file: str | Path,
               input_file: str | Path,
               width: int | None,
               scale: float | None = None,
               resampling: Image.Resampling = Image.Resampling.BICUBIC,
               ):
        """
        Resize input file.

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param width: Width in pixels.
        :param scale: Scale image in proportion. Must be >0 (if used).
        :param resampling: Resampling algorithm used.

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        with self._open(input_file) as img:
            if scale:
                if scale <= 0:
                    raise RuntimeError(_("Scale must be >0"))
                width = int(scale * img.width)
            if not width:
                raise RuntimeError(_("Need either scale or width to resize an image"))
            if width <= 0:
                raise RuntimeError(_("Width must be > 0"))
            height = int(width * float(img.height) / img.width)

            img = img.resize(
                size=(width, height),
                resample=resampling
            )
            self._save(
                img,
                output_file,
                format=format,
                quality=90,
                optimize=True,
            )

    def convert(
        self,
        output_file: str | Path,
        input_file: str | Path,
        quality: int = 90,
        optimize: bool = True,
    ):
        """
        Convert input file into an output.

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param quality: Final quality of image file (1-100). If 100, activates lossless compression. Valid only for JPG, WEBP out formats. Defaults to 90.
        :param optimize: Improve file size, without losing quality (lossless compression). Valid only for JPG, PNG, WEBP out formats Defaults to True.

        :raises ValueError: invalid quality value. Valid values are 1-100.
        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)
        if quality < 1 or quality > 100:
            raise ValueError(f"{_('Invalid quality level. Valid values are')} 1-100.")

        output_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[output_ext]["format"]

        img = self._open(input_file)
        self._save(
            img,
            output_file,
            format=format,
            quality=quality,
            optimize=optimize,
            lossless=True if quality == 100 else False,  # valid only for WEBP
        )

    def rotate(
        self,
            output_file: str | Path,
            input_file: str | Path,
            rotate: int,
            resampling: Image.Resampling = Image.Resampling.BICUBIC,
    ):
        """
        Rotate input file by X degrees (clockwise).

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param rotate: Rotation degrees (-360-360).
        :param resampling: Resampling algorithm used.

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        # parse rotation argument
        rotate = normalize_degree(rotate)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        img = img.rotate(-rotate, resample=resampling, expand=True)  # clockwise rotation
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def mirror(self, output_file: str | Path, input_file: str | Path, x_y: bool):
        """
        Mirror input file in relation X or Y axis.

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param x_y: Mirror in relation to x or y axis. True for X axis (mirror image horizontally). False for Y axis (flip image vertically).

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        if x_y:
            img = ImageOps.mirror(img)
        else:
            img = ImageOps.flip(img)
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def blur(
        self,
        output_file: str | Path,
        input_file: str | Path,
        blur_pixels: int,
    ):
        """
        Blurs an input image file using GaussianBlur.

        :param output_file: Output image file.
        :param input_file: Input image file.        
        :param blur_pixels: Blur radius (in pixels). Higher number = more blur.        

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        img = img.filter(
            ImageFilter.GaussianBlur(radius=blur_pixels)
        )
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def unsharp_mask(
        self,
        output_file: str | Path,
        input_file: str | Path,
        radius: float,
        percent: int,
        threshold: int,
    ):
        """
        Sharpens an input image file using unsharp mask.

        :param output_file: Output image file.
        :param input_file: Input image file.        

        :param radius: Pixels to blur.
        :param percent: Strength of sharpening.
        :param threshold: How different pixels must be from neighbors to be sharpened (controls noise amplification).

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        img = img.filter(ImageFilter.UnsharpMask(
            radius=radius,
            percent=percent,
            threshold=threshold,
        ))
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def antialias(
        self,
        output_file: str | Path,
        input_file: str | Path,
        radius: int,
        algorithm: AntialiasAlgorithm = AntialiasAlgorithm.MEDIAN,
    ):
        """
        Applies antialias to an input image file using Median or Mode algorithms.

        :param output_file: Output image file.
        :param input_file: Input image file.        

        :param radius: Box radius (kernel size) to calculate pixel averaging.
        :param algorithm: Algorithm used. Available options are "median" (default, replaces each pixel with the median of its neighbors), "mode" (replaces each pixel with the most common (mode) pixel value in the neighborhood).

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        img = img.filter(algorithm.value(radius))
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def enhance(
        self,
        output_file: str | Path,
        input_file: str | Path,
        color_factor: float = 1.0,
        contrast_factor: float = 1.0,
        brightness_factor: float = 1.0,
        sharpness_factor: float = 1.0,
    ):
        """
        Enhances an input image file.

        :param output_file: Output image file.
        :param input_file: Input image file.        

        :param color_factor: 0.0 = black and white | 1.0 = original color | 2.0 very saturated
        :param contrast_factor: 0.0 = gray | 1.0 = original constrat | 2.0 strong contrast
        :param brightness_factor: 0.0 = black | 1.0 = original brightness | 2.0 very bright
        :param sharpness_factor: 0.0 = blurred | 1.0 = original | 2.0 sharpen edges

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        img = ImageEnhance.Color(img).enhance(color_factor)
        img = ImageEnhance.Contrast(img).enhance(contrast_factor)
        img = ImageEnhance.Brightness(img).enhance(brightness_factor)
        img = ImageEnhance.Sharpness(img).enhance(sharpness_factor)
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def filter(
        self,
        output_file: str | Path,
        input_file: str | Path,
        filters: list[ImageFilter.BuiltinFilter] | list[str],
    ):
        """
        Enhances an input image file.

        :param output_file: Output image file.
        :param input_file: Input image file.        
        :param filters: Filters to apply in image. Check PillowFilter for supported filters.

        filters =

        - "blur": {_('Blurs image')}

        - "smooth": {_('Softens image, similar to blur')}

        - "smooth_more": {_('Softens image more')}

        - "sharpen": {_('Increase image sharpness')}

        - "sharpen_more": {_('Increase image sharpness - stronger sharpness')}

        - "edge_enhance": {_('Enhance edge contours of the image')}

        - "edge_enhance_more": {_('Enhance edge contours of the image - stronger enhancement')}

        - "edge_enhance_map": {_('Enhance edge contours of the image - use mapping algorithm')}

        - "emboss": {_('Create 3D emboss effect in the image')}

        - "emboss_draw": {_('Draw edge contours of the image')}

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = self._open(input_file)
        for filter in filters:
            if isinstance(filter, str):
                filter = self.PILLOW_FILTERS[filter]
            img = img.filter(filter)
        self._save(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def _open(self, input_file: Path | str):
        img = Image.open(input_file)
        # 1. Transparency -> convert to RGBA
        if img.mode == "P" and "transparency" in img.info:
            img = img.convert("RGBA")
        return img

    def _save(self, img: Image.Image, output_file: str | Path, format: str, **params):
        """
        Corrects common errors in images and saves them.

        :param img: Image to be corrected.
        :param output_file: File to save img.
        :param format: Target format to save img.
        :param params: Additional parameters for saving the image, such as quality and optimize.

        :raises Exception: if image correction fails.
        """
        try:
            format = format.upper()  # ensure uppercase format

            # 0. Preserve EXIF and ICC if they exist
            if "exif" in img.info and img.info["exif"]:
                params.setdefault("exif", img.info["exif"])
            if "icc_profile" in img.info and img.info["icc_profile"]:
                params.setdefault("icc_profile", img.info["icc_profile"])

            # 2. Convert incompatible modes to the target format
            if format in ("JPEG",) and img.mode not in ("RGB", "L"):
                img = img.convert("RGB")
            elif format in ("PNG", "WEBP") and img.mode not in ("RGB", "RGBA"):
                img = img.convert("RGBA")
            elif format == "TIFF" and img.mode not in ("RGB", "RGBA", "L"):
                img = img.convert("RGB")
            elif format == "BMP" and img.mode not in ("RGB",):
                img = img.convert("RGB")
        except Exception as e:
            logger.error(f"{_('Image correction failed')}: {e}")
            raise

        # save image
        img.save(output_file, format=format, **params)



# src\file_conversor\cli\doc\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.doc._typer import COMMAND_NAME

from file_conversor.cli.doc.convert_cmd import typer_cmd as convert_cmd

_ = get_translation()


doc_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=f"{_('Document file manipulation')} {_('(requires MS Office / LibreOffice)')})",
)
doc_cmd.add_typer(convert_cmd)




# src\file_conversor\cli\video\_ffmpeg_cmd.py

from rich import print

from typing import Annotated, Any, Callable, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend, FFprobeBackend
from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter, FFmpegFilterDeshake, FFmpegFilterEq, FFmpegFilterHflip, FFmpegFilterMInterpolate, FFmpegFilterScale, FFmpegFilterTranspose, FFmpegFilterUnsharp, FFmpegFilterVflip

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.formatters import parse_bytes
from file_conversor.utils.validators import check_valid_options

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def ffmpeg_cli_cmd(  # pyright: ignore[reportUnusedFunction]
    input_files: List[Path],

    file_format: str,
    out_stem: str = "",

    audio_bitrate: int = 0,
    video_bitrate: int = 0,

    audio_codec: str | None = None,
    video_codec: str | None = None,

    target_size: str | None = None,
    video_encoding_speed: str | None = None,
    video_quality: str | None = None,

    resolution: str | None = None,
    fps: int | None = None,

    brightness: float = 1.0,
    contrast: float = 1.0,
    color: float = 1.0,
    gamma: float = 1.0,

    rotation: int | None = None,
    mirror_axis: str | None = None,
    deshake: bool = False,
    unsharp: bool = False,

    output_dir: Path = Path(),
    progress_callback: Callable[[float, ProgressManager], Any] | None = None,
):
    # init ffmpeg
    ffmpeg_backend = FFmpegBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # target size
    target_size_bytes = 0
    if target_size and video_bitrate <= 0:
        ffprobe_backend = FFprobeBackend(
            install_deps=CONFIG['install-deps'],
            verbose=STATE["verbose"],
        )
        target_size_bytes = parse_bytes(target_size)

    # set filters
    audio_filters: list[FFmpegFilter] = list()
    video_filters: list[FFmpegFilter] = list()

    if resolution is not None:
        video_filters.append(FFmpegFilterScale(*resolution.split(":")))

    if fps is not None:
        video_filters.append(FFmpegFilterMInterpolate(fps=fps))

    if brightness != 1.0 or contrast != 1.0 or color != 1.0 or gamma != 1.0:
        video_filters.append(FFmpegFilterEq(brightness=brightness, contrast=contrast, saturation=color, gamma=gamma))

    if rotation is not None:
        if rotation in (90, -90):
            direction = {90: 1, -90: 2}[rotation]
            video_filters.append(FFmpegFilterTranspose(direction=direction))
        else:
            video_filters.append(FFmpegFilterTranspose(direction=1))
            video_filters.append(FFmpegFilterTranspose(direction=1))

    if mirror_axis is not None:
        if mirror_axis == "x":
            video_filters.append(FFmpegFilterHflip())
        else:
            video_filters.append(FFmpegFilterVflip())

    if deshake:
        video_filters.append(FFmpegFilterDeshake())

    if unsharp:
        video_filters.append(FFmpegFilterUnsharp())

    two_pass = (video_bitrate > 0) or (audio_bitrate > 0)

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        nonlocal audio_bitrate, video_bitrate, target_size_bytes
        logger.debug(f"Input file: {input_file}")

        # target size
        if target_size_bytes > 0:
            duration = ffprobe_backend.get_duration(input_file)
            if duration < 0:
                raise RuntimeError(_('Could not determine input file duration'))

            # total size in kbit
            target_size_kbit = int(target_size_bytes * 8.0 / 1024.0)
            target_size_kbps = int(target_size_kbit / duration)

            # audio size
            audio_bitrate = 128 if audio_bitrate <= 0 else audio_bitrate
            audio_kBps = audio_bitrate / 8.0
            audio_MB = audio_kBps * duration / 1024.0

            video_bitrate = target_size_kbps - audio_bitrate
            if video_bitrate < 1:
                raise RuntimeError(f"{_('Target size too small')}: {target_size}. {_(f'Increase target size to at least')} '{audio_MB + 0.100:.2f}M' {_('(might not be enougth to achieve good video quality)')}.")

        logger.debug(f"{_('Audio bitrate')}: [green][bold]{audio_bitrate} kbps[/bold][/green]")
        logger.debug(f"{_('Video bitrate')}: [green][bold]{video_bitrate} kbps[/bold][/green]")

        ffmpeg_backend.set_files(input_file=input_file, output_file=output_file)
        ffmpeg_backend.set_audio_codec(
            codec=audio_codec,
            bitrate=audio_bitrate,
            filters=audio_filters,
        )
        ffmpeg_backend.set_video_codec(
            codec=video_codec,
            bitrate=video_bitrate,
            filters=video_filters,
            encoding_speed=video_encoding_speed,
            quality_setting=video_quality,
        )

        progress_update_cb = progress_mgr.update_progress
        if progress_callback is not None:
            def progress_update_cb(step_progress: float): return progress_callback(step_progress, progress_mgr)  # pyright: ignore[reportOptionalCall]

        progress_complete_cb = progress_mgr.complete_step
        if progress_callback is not None:
            def progress_complete_cb(): return progress_callback(progress_mgr.complete_step(), progress_mgr)  # pyright: ignore[reportOptionalCall]

        # display current progress
        process = ffmpeg_backend.execute(
            progress_callback=progress_update_cb,
            pass_num=1 if two_pass else 0,
        )
        progress_complete_cb()

        if two_pass:
            # display current progress
            process = ffmpeg_backend.execute(
                progress_callback=progress_update_cb,
                pass_num=2,
            )
            progress_complete_cb()

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, steps=2 if two_pass else 1, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{file_format}", out_stem=out_stem)

    logger.info(f"{_('FFMpeg result')}: [green][bold]{_('SUCCESS')}[/bold][/green]")




# src\file_conversor\config\locale.py

import gettext  # app translations / locales
import locale
import pycountry

from babel import Locale
from rich import print
from typing import Callable

from file_conversor.config.environment import Environment
from file_conversor.config.config import Configuration

CONFIG = Configuration.get_instance()

AVAILABLE_LANGUAGES = set([str(mo.relative_to(Environment.get_locales_folder()).parts[0]) for mo in Environment.get_locales_folder().glob("**/LC_MESSAGES/*.mo")])

_gettext_instance: Callable[[str], str] | None = None


def _get_lang_name_babel(lang_code: str) -> str:
    """
    Get language name using babel library.

    :param lang_code: Language code (e.g., "en", "fr", "es", "eng", "en_US")

    :return: Language name (e.g., "English", "Français", "Español")
    :raises Exception: If language code is invalid or name not found.
    """
    locale = Locale.parse(lang_code)
    try:
        display_lang = normalize_lang_code(CONFIG["language"])[:2].lower()
        lang = locale.get_display_name(display_lang)
        if not lang:
            raise ValueError(f"No language name found for display language '{display_lang}'.")
    except:
        display_lang = get_default_language()[:2].lower()
        lang = locale.get_display_name(display_lang)
        if not lang:
            raise ValueError(f"No language name found for display language '{display_lang}'.")
    return lang


def _get_lang_name_pycountry(lang_code: str) -> str:
    """
    Get language name using pycountry library.

    :param lang_code: Language code (e.g., "en", "fr", "es", "eng", "en_US")
    :return: Language name (e.g., "English", "Français", "Español")
    :raises Exception: If language code is invalid or name not found.
    """
    CUSTOM_LANG_ALIASES = {
        "chi_sim": "Chinese (Simplified)",
        "chi_tra": "Chinese (Traditional)",
        "chi_sim_vert": "Chinese (Simplified, Vertical)",
        "chi_tra_vert": "Chinese (Traditional, Vertical)",
        "equ": "Mathematical Equations (OCR)",
        "osd": "Orientation & Script Detection",
    }

    base_code = lang_code.split('_')[0]  # remove region part like en_US → en
    lang = pycountry.languages.get(alpha_3=base_code)
    if not lang:
        lang = pycountry.languages.get(alpha_2=base_code)
    if lang and hasattr(lang, 'name'):
        return str(lang.name)
    if lang_code in CUSTOM_LANG_ALIASES:
        return CUSTOM_LANG_ALIASES[lang_code]
    raise ValueError(f"No language name found for code '{lang_code}'.")


def _print_debug():
    print(f"Locales folder: {Environment.get_locales_folder()}")
    print(f"Available languages: {sorted(AVAILABLE_LANGUAGES)} ({len(AVAILABLE_LANGUAGES)} entries)")
    print(f"Config / sys lang: ({CONFIG['language']} / {get_system_locale()})")


def get_default_language():
    return "en_US"


def normalize_lang_code(lang: str | None) -> str:
    if not lang or lang not in AVAILABLE_LANGUAGES:
        return ""  # empty language code (force fallback in translation)
    return lang


# Get translations
def get_system_locale():
    """Get system default locale"""
    lang, _ = locale.getlocale()
    return lang


def get_translation():
    """
    Get translation mechanism, based on user preferences.
    """
    global _gettext_instance
    if _gettext_instance:
        return _gettext_instance

    languages: list[str] = []
    try:
        languages = [
            normalize_lang_code(CONFIG["language"]),
            normalize_lang_code(get_system_locale()),
            normalize_lang_code(get_default_language()),  # fallback
        ]
        languages = [lang for lang in languages if lang]  # Filter out None entries
        if not languages:
            print(f"WARNING: No valid languages found")
            _print_debug()
        translation = gettext.translation(
            'messages', Environment.get_locales_folder(),
            languages=languages,
            fallback=True,
        )
        _gettext_instance = translation.gettext
        return _gettext_instance
    except:
        _print_debug()
        print(f"Languages tried: {languages}")
        raise


def get_language_name(lang_code: str) -> str:
    """
    Get language name from code.

    :param lang_code: Language code (e.g., "en", "fr", "es", "eng", "en_US")

    :return: Language name (e.g., "English", "Français", "Español")
    """
    lang_code = lang_code.strip().replace('-', '_')

    try:
        return _get_lang_name_babel(lang_code).title()
    except Exception as e:
        pass
    try:
        return _get_lang_name_pycountry(lang_code).title()
    except Exception as e:
        print(f"[bold]WARNING[/]: Could not parse language code '{lang_code}'. Falling back to '{lang_code}'.")
        return lang_code


__all__ = [
    "AVAILABLE_LANGUAGES",
    "get_default_language",
    "normalize_lang_code",
    "get_system_locale",
    "get_translation",
    "get_language_name",
]
